@@ -5,13 +5,13 @@ Features that are implemented
 
  * capstyles and join styles
  * dashes
- * linewidth 
+ * linewidth
  * lines, rectangles, ellipses
  * clipping to a rectangle
  * output to RGBA and PNG
  * alpha blending
  * DPI scaling properly - everything scales properly (dashes, linewidths, etc)
- * draw polygon 
+ * draw polygon
  * freetype2 w/ ft2font
 
 TODO:
@@ -21,8 +21,8 @@ TODO:
   * allow load from png
 
   * integrate screen dpi w/ ppi and text
-  
-INSTALLING 
+
+INSTALLING
 
   REQUIREMENTs
 
@@ -32,7 +32,7 @@ INSTALLING
     freetype 2
     libpng
     libz
-    
+
   Install AGG2 (cut and paste below into xterm should work)
 
     wget http://www.antigrain.com/agg2.tar.gz
@@ -46,7 +46,7 @@ INSTALLING
 
   Installing backend_agg
 
-     
+
    Edit setup.py: change aggsrc to point to the agg2 src tree and
    replace if 0: with if 1: in the backend_agg section
 
@@ -59,19 +59,20 @@ INSTALLING
 
   Using agg backend
 
-    python somefile.py -dAgg   
+    python somefile.py -dAgg
 
   or
 
     import matplotlib
     matplotlib.use('Agg')
-    
+
 
 """
 from __future__ import division
 
 import os, sys
-from matplotlib import verbose
+import matplotlib
+from matplotlib import verbose, rcParams
 from matplotlib.numerix import array, Float, zeros, transpose
 from matplotlib._image import fromarray
 from matplotlib._pylab_helpers import Gcf
@@ -83,15 +84,16 @@ from matplotlib.figure import Figure
 from matplotlib.font_manager import fontManager
 from matplotlib.ft2font import FT2Font
 from matplotlib.mathtext import math_parse_s_ft2font
-from matplotlib.texmanager import TexManager
 from matplotlib.transforms import lbwh_to_bbox
 from matplotlib.numerix.mlab import fliplr
 import matplotlib.numerix
 
 if matplotlib.numerix.which[0] == "numarray":
     from _na_backend_agg import RendererAgg as _RendererAgg
-else:
+elif matplotlib.numerix.which[0] == "numeric":
     from _nc_backend_agg import RendererAgg as _RendererAgg
+else:
+    from _ns_backend_agg import RendererAgg as _RendererAgg
 
 backend_version = 'v2.2'
 _fontd = {}     # a map from fname to font instances
@@ -102,16 +104,21 @@ class RendererAgg(RendererBase):
     The renderer handles all the drawing primitives using a graphics
     context instance that controls the colors/styles
     """
-    
+
     debug=1
     texd = {}  # a cache of tex image rasters
     def __init__(self, width, height, dpi):
         if __debug__: verbose.report('RendererAgg.__init__', 'debug-annoying')
+        RendererBase.__init__(self)
         self.dpi = dpi
         self.width = width
         self.height = height
+        if __debug__: verbose.report('RendererAgg.__init__ width=%s, \
+                        height=%s'%(width, height), 'debug-annoying')
         self._renderer = _RendererAgg(int(width), int(height), dpi.get(),
                                     debug=False)
+        if __debug__: verbose.report('RendererAgg.__init__ _RendererAgg done',
+                                     'debug-annoying')
         self.draw_polygon = self._renderer.draw_polygon
         self.draw_rectangle = self._renderer.draw_rectangle
         self.draw_path = self._renderer.draw_path
@@ -119,19 +126,19 @@ class RendererAgg(RendererBase):
         self.draw_markers = self._renderer.draw_markers
         self.draw_image = self._renderer.draw_image
         self.draw_line_collection = self._renderer.draw_line_collection
+        self.draw_quad_mesh = self._renderer.draw_quad_mesh
         self.draw_poly_collection = self._renderer.draw_poly_collection
         self.draw_regpoly_collection = self._renderer.draw_regpoly_collection
 
         self.copy_from_bbox = self._renderer.copy_from_bbox
-        self.restore_region = self._renderer.restore_region        
-        
+        self.restore_region = self._renderer.restore_region
 
-        self.texmanager = TexManager()
 
         self.bbox = lbwh_to_bbox(0,0, self.width, self.height)
-        
+        if __debug__: verbose.report('RendererAgg.__init__ done',
+                                     'debug-annoying')
 
-    def draw_arc(self, gcEdge, rgbFace, x, y, width, height, angle1, angle2):
+    def draw_arc(self, gcEdge, rgbFace, x, y, width, height, angle1, angle2, rotation):
         """
         Draw an arc centered at x,y with width and height and angles
         from 0.0 to 360.0
@@ -144,17 +151,8 @@ class RendererAgg(RendererBase):
         """
         if __debug__: verbose.report('RendererAgg.draw_arc', 'debug-annoying')
         self._renderer.draw_ellipse(
-            gcEdge, rgbFace, x, y, width/2, height/2)  # ellipse takes radius
-        
+            gcEdge, rgbFace, x, y, width/2, height/2, rotation)  # ellipse takes radius
 
-    def _draw_image(self, x, y, im):
-        """
-        Draw the Image instance into the current axes; x, y is the
-        upper left hand corner of the image
-        """
-        if __debug__: verbose.report('RendererAgg.draw_image', 'debug-annoying')
-        #self._renderer.draw_image(int(x), int(self.height-y), im)
-        self._renderer.draw_image(int(x), int(y), im)        
 
     def draw_line(self, gc, x1, y1, x2, y2):
         """
@@ -174,25 +172,26 @@ class RendererAgg(RendererBase):
         if __debug__: verbose.report('RendererAgg.draw_point', 'debug-annoying')
         rgbFace = gc.get_rgb()
         self._renderer.draw_ellipse(
-            gc, rgbFace, x, y, 0.5, 0.5)        
-
-    def draw_mathtext(self, gc, x, y, s, prop, angle):    
+            gc, rgbFace, x, y, 0.5, 0.5, 0.0)
+            
+    def draw_mathtext(self, gc, x, y, s, prop, angle):
         """
         Draw the math text using matplotlib.mathtext
         """
-        if __debug__: verbose.report('RendererAgg.draw_mathtext', 'debug-annoying')
+        if __debug__: verbose.report('RendererAgg.draw_mathtext',
+                                     'debug-annoying')
         size = prop.get_size_in_points()
         width, height, fonts = math_parse_s_ft2font(
             s, self.dpi.get(), size, angle)
-        
+
         if angle == 90:
             width, height = height, width
         for font in fonts:
-            if angle == 90:             
+            if angle == 90:
                 font.horiz_image_to_vert_image() # <-- Rotate
                 self._renderer.draw_text( font, int(x)-width, int(y)-height, gc)
             else:
-                self._renderer.draw_text( font, int(x), int(y)-height, gc)                
+                self._renderer.draw_text( font, int(x), int(y)-height, gc)
         if 0:
             self._renderer.draw_rectangle(gc, None,
                                           int(x),
@@ -200,7 +199,7 @@ class RendererAgg(RendererBase):
                                           width, height)
 
 
-    
+
     def draw_text(self, gc, x, y, s, prop, angle, ismath):
         """
         Render the text
@@ -220,9 +219,9 @@ class RendererAgg(RendererBase):
         font.draw_glyphs_to_bitmap()
 
         #print x, y, int(x), int(y)
-        
+
         self._renderer.draw_text(font, int(x), int(y), gc)
-        
+
 
     def get_text_width_height(self, s, prop, ismath, rgb=(0,0,0)):
         """
@@ -237,11 +236,11 @@ class RendererAgg(RendererBase):
         if ismath=='TeX':
             # todo: handle props
             size = prop.get_size_in_points()
-            dpi = self.dpi.get()
-            Z = self.texmanager.get_rgba(s, size, dpi, rgb)
+            texmanager = self.get_texmanager()
+            Z = texmanager.get_rgba(s, size, self.dpi.get(), rgb)
             m,n,tmp = Z.shape
             return n,m
-        
+
         if ismath:
             width, height, fonts = math_parse_s_ft2font(
                 s, self.dpi.get(), prop.get_size_in_points())
@@ -265,11 +264,11 @@ class RendererAgg(RendererBase):
             w,h = h,w
             x -= w
 
-        key = s, size, dpi, rgb
+        texmanager = self.get_texmanager()
+        key = s, size, dpi, rgb, angle, texmanager.get_font_config()
         im = self.texd.get(key)
-        
         if im is None:
-            Z = self.texmanager.get_rgba(s, size, dpi, rgb)
+            Z = texmanager.get_rgba(s, size, dpi, rgb)
             if flip:
                 r = Z[:,:,0]
                 g = Z[:,:,1]
@@ -286,10 +285,14 @@ class RendererAgg(RendererBase):
                 Z[:,:,2] = func(b)
                 Z[:,:,3] = func(a)
             im = fromarray(Z, 1)
-            self.texd[key] = im            
+            im.flipud_out()
+            self.texd[key] = im
+
+        cliprect = gc.get_clip_rectangle()
+        if cliprect is None: bbox = None
+        else: bbox = lbwh_to_bbox(*cliprect)
+        self.draw_image(x, self.height-y, im, bbox)
 
-        self.draw_image(x, y-h, im, 'upper', self.bbox)
-        
     def get_canvas_width_height(self):
         'return the canvas width and height in display coords'
         return self.width, self.height
@@ -299,11 +302,12 @@ class RendererAgg(RendererBase):
         """
         Get the font for text instance t, cacheing for efficiency
         """
-        if __debug__: verbose.report('RendererAgg._get_agg_font', 'debug-annoying')
+        if __debug__: verbose.report('RendererAgg._get_agg_font',
+                                     'debug-annoying')
 
         key = hash(prop)
         font = _fontd.get(key)
-        
+
         if font is None:
             fname = fontManager.findfont(prop)
             font = FT2Font(str(fname))
@@ -321,33 +325,40 @@ class RendererAgg(RendererBase):
         convert point measures to pixes using dpi and the pixels per
         inch of the display
         """
-        if __debug__: verbose.report('RendererAgg.points_to_pixels', 'debug-annoying')
+        if __debug__: verbose.report('RendererAgg.points_to_pixels',
+                                     'debug-annoying')
         return points*self.dpi.get()/72.0
 
     def tostring_rgb(self):
-        if __debug__: verbose.report('RendererAgg.tostring_rgb', 'debug-annoying')
+        if __debug__: verbose.report('RendererAgg.tostring_rgb',
+                                     'debug-annoying')
         return self._renderer.tostring_rgb()
 
     def tostring_argb(self):
-        if __debug__: verbose.report('RendererAgg.tostring_argb', 'debug-annoying')
+        if __debug__: verbose.report('RendererAgg.tostring_argb',
+                                     'debug-annoying')
         return self._renderer.tostring_argb()
-        
-    def buffer_rgba(self):
-        if __debug__: verbose.report('RendererAgg.buffer_rgba', 'debug-annoying')
-        return self._renderer.buffer_rgba()
-        
+
+    def buffer_rgba(self,x,y):
+        if __debug__: verbose.report('RendererAgg.buffer_rgba',
+                                     'debug-annoying')
+        return self._renderer.buffer_rgba(x,y)
+
     def clear(self):
         self._renderer.clear()
-        
 
-    
+
+
 def new_figure_manager(num, *args, **kwargs):
     """
     Create a new figure manager instance
     """
-    if __debug__: verbose.report('backend_agg.new_figure_manager', 'debug-annoying')
+    if __debug__: verbose.report('backend_agg.new_figure_manager',
+                                 'debug-annoying')
 
-    thisFig = Figure(*args, **kwargs)
+
+    FigureClass = kwargs.pop('FigureClass', Figure)
+    thisFig = FigureClass(*args, **kwargs)
     canvas = FigureCanvasAgg(thisFig)
     manager = FigureManagerBase(canvas, num)
     return manager
@@ -361,7 +372,7 @@ class FigureCanvasAgg(FigureCanvasBase):
     Public attribute
 
       figure - A Figure instance
-    """    
+    """
 
     def copy_from_bbox(self, bbox):
         renderer = self.get_renderer()
@@ -370,7 +381,7 @@ class FigureCanvasAgg(FigureCanvasBase):
     def restore_region(self, region):
         renderer = self.get_renderer()
         return renderer.restore_region(region)
-        
+
     def draw(self):
         """
         Draw the figure using the renderer
@@ -391,22 +402,24 @@ class FigureCanvasAgg(FigureCanvasBase):
             self.renderer = RendererAgg(w, h, self.figure.dpi)
             self._lastKey = key
         return self.renderer
-    
+
     def tostring_rgb(self):
-        if __debug__: verbose.report('FigureCanvasAgg.tostring_rgb', 'debug-annoying')
+        if __debug__: verbose.report('FigureCanvasAgg.tostring_rgb',
+                                     'debug-annoying')
         return self.renderer.tostring_rgb()
 
     def tostring_argb(self):
-        if __debug__: verbose.report('FigureCanvasAgg.tostring_argb', 'debug-annoying')
+        if __debug__: verbose.report('FigureCanvasAgg.tostring_argb',
+                                     'debug-annoying')
         return self.renderer.tostring_argb()
-        
-    def buffer_rgba(self):
-        if __debug__: verbose.report('FigureCanvasAgg.buffer_rgba', 'debug-annoying')
-        return self.renderer.buffer_rgba()
-    
-    def print_figure(self, filename, dpi=150,
-                     facecolor='w', edgecolor='w',
-                     orientation='portrait'):
+
+    def buffer_rgba(self,x,y):
+        if __debug__: verbose.report('FigureCanvasAgg.buffer_rgba',
+                                     'debug-annoying')
+        return self.renderer.buffer_rgba(x,y)
+
+    def print_figure(self, filename, dpi=None, facecolor='w', edgecolor='w',
+                     orientation='portrait', **kwargs):
         """
         Render the figure to hardcopy.  Set the figure patch face and
         edge colors.  This is useful because some of the GUIs have a
@@ -420,9 +433,9 @@ class FigureCanvasAgg(FigureCanvasBase):
         If filename is a fileobject, write png to file object (thus
         you can, for example, write the png to stdout
         """
-        if __debug__: verbose.report('FigureCanvasAgg.print_figure', 'debug-annoying')
-
-            
+        if __debug__: verbose.report('FigureCanvasAgg.print_figure',
+                                     'debug-annoying')
+        if dpi is None: dpi = matplotlib.rcParams['savefig.dpi']
 
         # store the orig figure dpi, color and size information so we
         # can restore them later.  For image creation alone, this is
@@ -435,7 +448,6 @@ class FigureCanvasAgg(FigureCanvasBase):
         origfacecolor = self.figure.get_facecolor()
         origedgecolor = self.figure.get_edgecolor()
 
-
         self.figure.dpi.set(dpi)
         self.figure.set_facecolor(facecolor)
         self.figure.set_edgecolor(edgecolor)
@@ -443,6 +455,7 @@ class FigureCanvasAgg(FigureCanvasBase):
         # render the printed figure
         self.draw()
 
+        printfunc = None
         if not is_string_like(filename):
             # assume png and write to fileobject
             self.renderer._renderer.write_png(filename)
@@ -453,7 +466,7 @@ class FigureCanvasAgg(FigureCanvasBase):
             if not len(ext):
                 ext = '.png'
                 filename += ext
- 
+
             ext = ext.lower()
             if (ext.find('rgb')>=0 or
                 ext.find('raw')>=0 or
@@ -467,16 +480,33 @@ class FigureCanvasAgg(FigureCanvasBase):
             elif ext.find('svg')>=0:
                 from backend_svg import FigureCanvasSVG
                 svg = self.switch_backends(FigureCanvasSVG)
-                svg.print_figure(filename, dpi, facecolor, edgecolor, orientation)
+                printfunc = svg.print_figure
             elif ext.find('ps')>=0 or ext.find('ep')>=0:
                 from backend_ps import FigureCanvasPS # lazy import
                 ps = self.switch_backends(FigureCanvasPS)
-                ps.print_figure(filename, dpi, facecolor, edgecolor, orientation)
+                printfunc = ps.print_figure
+            elif ext.find('pdf')>=0:
+                from backend_pdf import FigureCanvasPdf
+                pdf = self.switch_backends(FigureCanvasPdf)
+                printfunc = pdf.print_figure
             else:
                 raise IOError('Do not know know to handle extension *%s' % ext)
 
+        if printfunc is not None:
+            try:
+                printfunc(filename, dpi, facecolor, edgecolor, orientation, **kwargs)
+            except:
+                # restore the original figure properties
+                self.figure.dpi.set(origDPI)
+                self.figure.set_facecolor(origfacecolor)
+                self.figure.set_edgecolor(origedgecolor)
+                self.figure.set_canvas(self)            
+                raise
+
         # restore the original figure properties
         self.figure.dpi.set(origDPI)
         self.figure.set_facecolor(origfacecolor)
         self.figure.set_edgecolor(origedgecolor)
         self.figure.set_canvas(self)
+            
+            
