@@ -1,16 +1,16 @@
 """
 A Cairo backend for matplotlib
 Author: Steve Chaplin
- 
+
 Cairo is a vector graphics library with cross-device output support.
 Features of Cairo:
  * anti-aliasing
- * alpha channel 
+ * alpha channel
  * saves image files as PNG, PostScript, PDF
 
 http://cairographics.org
 Requires (in order, all available from Cairo website):
-    libpixman, cairo, pycairo
+    cairo, pycairo
 
 Naming Conventions
   * classes MixedUpperCase
@@ -24,7 +24,15 @@ import sys
 import warnings
 def _fn_name(): return sys._getframe(1).f_code.co_name
 
-import matplotlib.agg as agg
+import cairo
+_version_required = (1,2,0)
+if cairo.version_info < _version_required:
+   raise SystemExit ("Pycairo %d.%d.%d is installed\n"
+                     "Pycairo %d.%d.%d or later is required"
+                     % (cairo.version_info + _version_required))
+backend_version = cairo.version
+del _version_required
+
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
      FigureManagerBase, FigureCanvasBase
 from matplotlib.cbook      import enumerate, izip
@@ -33,30 +41,20 @@ from matplotlib.mathtext   import math_parse_s_ft2font
 import matplotlib.numerix as numx
 from matplotlib.transforms import Bbox
 
-import cairo
-
-_version_required = (0,5,0)
-if cairo.version_info < _version_required:
-   raise SystemExit ("Pycairo %d.%d.%d is installed\n"
-                     "Pycairo %d.%d.%d or later is required"
-                     % (cairo.version_info + _version_required))
-backend_version = cairo.version
-del _version_required
-
-if hasattr (cairo.ImageSurface, 'create_for_array'):
-   HAVE_CAIRO_NUMPY = True
-else:
-   HAVE_CAIRO_NUMPY = False
-
 
 _debug = False
 #_debug = True
 
 # Image formats that this backend supports - for print_figure()
-IMAGE_FORMAT          = ['eps', 'png', 'ps', 'svg']
-#IMAGE_FORMAT          = ['eps', 'pdf', 'png', 'ps', 'svg'] # pdf not ready yet
+IMAGE_FORMAT          = ['eps', 'pdf', 'png', 'ps', 'svg']
 IMAGE_FORMAT_DEFAULT  = 'png'
 
+# Image::color_conv(format) for draw_image()
+if sys.byteorder == 'little':
+   BYTE_FORMAT = 0 # BGRA
+else:
+   BYTE_FORMAT = 1 # ARGB
+
 
 class RendererCairo(RendererBase):
     fontweights = {
@@ -84,30 +82,22 @@ class RendererCairo(RendererBase):
         'normal'  : cairo.FONT_SLANT_NORMAL,
         'oblique' : cairo.FONT_SLANT_OBLIQUE,
         }
-    
+
 
     def __init__(self, dpi):
         """
         """
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        self.dpi      = dpi
-        surface = cairo.ImageSurface (cairo.FORMAT_ARGB32,1,1)
-        self.text_ctx = cairo.Context (surface)
+        self.dpi = dpi
+        self.text_ctx = cairo.Context (
+           cairo.ImageSurface (cairo.FORMAT_ARGB32,1,1))
 
 
     def set_ctx_from_surface (self, surface):
-        self.ctx = cairo.Context (surface)
+       self.ctx = cairo.Context (surface)
+       self.ctx.save() # restore, save  - when call new_gc()
 
 
-    def set_ctx_from_pixmap (self, pixmap): 
-        # not used by image backend (backend_cairo),
-        # uses _set_ctx_from_surface() instead
-        # used by GUI backend (backend_gtk)
-        self.ctx = cairo.gtk.gdk_cairo_create (pixmap)
-        
-    set_pixmap = set_ctx_from_pixmap
-    
-
     def set_width_height(self, width, height):
         self.width  = width
         self.height = height
@@ -116,52 +106,65 @@ class RendererCairo(RendererBase):
         # - problem with text? - will need to switch matrix_flipy off, or do a
         # font transform?
 
-    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2):
+
+    def _fill_and_stroke (self, ctx, fill_c):
+        #assert fill_c or stroke_c
+
+        #_.ctx.save()
+
+        if fill_c:
+            ctx.save()
+            ctx.set_source_rgb (*fill_c)
+            #if stroke_c:   # always (implicitly) set at the moment
+            ctx.fill_preserve()
+            #else:
+            #    ctx.fill()
+            ctx.restore()
+
+        #if stroke_c:                      # always stroke
+            #ctx.set_source_rgb (stroke_c) # is already set
+        ctx.stroke()
+
+        #_.ctx.restore() # revert to the default attributes
+
+
+    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2,
+                 rotation):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        # draws circular arcs where width=height
+        # this implementation draws circular arcs where width=height
         # FIXME
         # to get a proper arc of width/height you can use translate() and
-        # scale(), see draw_arc() manual page
+        # scale(), see cairo_arc() manual page
+
+        ctx = gc.ctx
+        ctx.save()
+        ctx.new_path()  # prevent cairo_arc() adding to current path
+        ctx.rotate(rotation)
         radius = (height + width) / 4
-        ctx    = gc.ctx
-        ctx.new_path()
         ctx.arc (x, self.height - y, radius,
                  angle1 * numx.pi/180.0, angle2 * numx.pi/180.0)
+        ctx.restore()
+        self._fill_and_stroke (ctx, rgbFace)
 
-        if rgbFace:
-            ctx.save()
-            ctx.set_source_rgb (*rgbFace)
-            ctx.fill_preserve()
-            ctx.restore()
-        ctx.stroke()
-    
-    
-    def draw_image(self, x, y, im, origin, bbox):
+
+    def draw_image(self, x, y, im, bbox):
         # bbox - not currently used
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
 
-        if numx.which[0] == "numarray":
-            warnings.warn("draw_image() currently works for numpy, but not "
-                          "numarray")
-            return
+        im.flipud_out()
 
-        if not HAVE_CAIRO_NUMPY:
-            warnings.warn("cairo with Numeric support is required for "
-                          "draw_image()")
-            return
-
-        #flipud = origin=='lower'             # gtk method (uses RGBA)
-        #rows, cols, s = im.as_str(flipud)
-        rows, cols, buf = im.buffer_argb32()  # ARGB32, but colors still wrong
-        X = numx.fromstring (buf, numx.UInt8)
-        X.shape = rows, cols, 4
+        rows, cols, buf = im.color_conv (BYTE_FORMAT)
+        surface = cairo.ImageSurface.create_for_data (
+                      buf, cairo.FORMAT_ARGB32, rows, cols, rows*4)
 
         # function does not pass a 'gc' so use renderer.ctx
         ctx = self.ctx
-        surface = cairo.ImageSurface.create_for_array (X)
+        y = self.height - y - rows
         ctx.set_source_surface (surface, x, y)
         ctx.paint()
-        
+
+        im.flipud_out()
+
 
     def draw_line(self, gc, x1, y1, x2, y2):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
@@ -169,7 +172,7 @@ class RendererCairo(RendererBase):
         ctx.new_path()
         ctx.move_to (x1, self.height - y1)
         ctx.line_to (x2, self.height - y2)
-        ctx.stroke()
+        self._fill_and_stroke (ctx, None)
 
 
     def draw_lines(self, gc, x, y, transform=None):
@@ -179,7 +182,7 @@ class RendererCairo(RendererBase):
             if transform.need_nonlinear():
                 x, y = transform.nonlinear_only_numerix(x, y)
             x, y = transform.numerix_x_y(x, y)
-        
+
         ctx = gc.ctx
         matrix_old = ctx.get_matrix()
         ctx.set_matrix (self.matrix_flipy)
@@ -191,19 +194,19 @@ class RendererCairo(RendererBase):
 
         for x,y in points:
             ctx.line_to (x, y)
-        ctx.stroke()
+        self._fill_and_stroke (ctx, None)
 
         ctx.set_matrix (matrix_old)
 
 
-    def draw_markers(self, gc, path, rgbFace, x, y, transform):
+    def draw_markers_OLD(self, gc, path, rgbFace, x, y, transform):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
 
         ctx = gc.ctx
 
         if transform.need_nonlinear():
             x,y = transform.nonlinear_only_numerix(x, y)
-        
+
         x, y = transform.numerix_x_y(x, y) # do nonlinear and affine transform
 
         # TODO - use cairo transform
@@ -232,25 +235,20 @@ class RendererCairo(RendererBase):
             ctx.translate(x, self.height - y)
             generate_path (path_list)
 
-            if rgbFace:
-               ctx.save()
-               ctx.set_source_rgb (*rgbFace)
-               # later - set alpha also?                     
-               ctx.fill_preserve()
-               ctx.restore() # undo colour change and restore path
-            
-            ctx.stroke()
+            self._fill_and_stroke (ctx, rgbFace)
+
             ctx.restore() # undo translate()
 
         #ctx.set_matrix(matrix_old)
 
-        
+
     def draw_point(self, gc, x, y):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
         # render by drawing a 0.5 radius circle
-        gc.ctx.new_path()
-        gc.ctx.arc (x, self.height - y, 0.5, 0, 2*numx.pi)
-        gc.ctx.fill()
+        ctx = gc.ctx
+        ctx.new_path()
+        ctx.arc (x, self.height - y, 0.5, 0, 2*numx.pi)
+        self._fill_and_stroke (ctx, gc.get_rgb())
 
 
     def draw_polygon(self, gc, rgbFace, points):
@@ -259,7 +257,7 @@ class RendererCairo(RendererBase):
         ctx = gc.ctx
         matrix_old = ctx.get_matrix()
         ctx.set_matrix (self.matrix_flipy)
-        
+
         ctx.new_path()
         x, y = points[0]
         ctx.move_to (x, y)
@@ -267,13 +265,8 @@ class RendererCairo(RendererBase):
             ctx.line_to (x, y)
         ctx.close_path()
 
-        if rgbFace:
-            ctx.save()
-            ctx.set_source_rgb (*rgbFace)
-            ctx.fill_preserve()
-            ctx.restore()
-        ctx.stroke()
-        
+        self._fill_and_stroke (ctx, rgbFace)
+
         ctx.set_matrix (matrix_old)
 
     def draw_rectangle(self, gc, rgbFace, x, y, width, height):
@@ -281,12 +274,7 @@ class RendererCairo(RendererBase):
         ctx = gc.ctx
         ctx.new_path()
         ctx.rectangle (x, self.height - y - height, width, height)
-        if rgbFace:
-            ctx.save()
-            ctx.set_source_rgb (*rgbFace)
-            ctx.fill_preserve()
-            ctx.restore()
-        ctx.stroke()
+        self._fill_and_stroke (ctx, rgbFace)
 
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath=False):
@@ -307,7 +295,7 @@ class RendererCairo(RendererBase):
 
            # size = prop.get_size_in_points() * self.dpi.get() / 96.0
            size = prop.get_size_in_points() * self.dpi.get() / 72.0
-        
+
            ctx.save()
            if angle:
               ctx.rotate (-angle * numx.pi / 180)
@@ -318,17 +306,17 @@ class RendererCairo(RendererBase):
 
     def _draw_mathtext(self, gc, x, y, s, prop, angle):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-       # mathtext using the gtk/gdk method
+        # mathtext using the gtk/gdk method
 
-        if numx.which[0] == "numarray":
-            warnings.warn("_draw_mathtext() currently works for numpy, but "
-                          "not numarray")
-            return
+        #if numx.which[0] == "numarray":
+        #   warnings.warn("_draw_mathtext() currently works for numpy, but "
+        #                 "not numarray")
+        #   return
 
-        if not HAVE_CAIRO_NUMPY:
-            warnings.warn("cairo with Numeric support is required for "
-                          "_draw_mathtext()")
-            return
+        #if not HAVE_CAIRO_NUMPY:
+        #    warnings.warn("cairo with Numeric support is required for "
+        #                  "_draw_mathtext()")
+        #    return
 
         size = prop.get_size_in_points()
         width, height, fonts = math_parse_s_ft2font(
@@ -349,7 +337,7 @@ class RendererCairo(RendererBase):
             if angle == 90:
                 font.horiz_image_to_vert_image() # <-- Rotate
             imw, imh, s = font.image_as_str()
-            Xall[:,i] = numx.fromstring(s, numx.UInt8)  
+            Xall[:,i] = numx.fromstring(s, numx.UInt8)
 
         # get the max alpha at each pixel
         Xs = numx.mlab.max (Xall,1)
@@ -364,12 +352,14 @@ class RendererCairo(RendererBase):
         pa[:,:,2] = int(rgb[2]*255)
         pa[:,:,3] = Xs
 
-        # works for numpy pa, not a numarray pa
-        surface = cairo.ImageSurface.create_for_array (pa)
+        ## works for numpy pa, not a numarray pa
+        #surface = cairo.ImageSurface.create_for_array (pa)
+        surface = cairo.ImageSurface.create_for_data (pa, cairo.FORMAT_ARGB32,
+                                                      imw, imh, imw*4)
         gc.ctx.set_source_surface (surface, x, y)
         gc.ctx.paint()
         #gc.ctx.show_surface (surface, imw, imh)
-            
+
 
     def flipy(self):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
@@ -377,11 +367,11 @@ class RendererCairo(RendererBase):
         #return False # tried - all draw objects ok except text (and images?)
         # which comes out mirrored!
 
-    
+
     def get_canvas_width_height(self):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
         return self.width, self.height
-    
+
 
     def get_text_width_height(self, s, prop, ismath):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
@@ -401,29 +391,29 @@ class RendererCairo(RendererBase):
 
         #size = prop.get_size_in_points() * self.dpi.get() / 96.0
         size = prop.get_size_in_points() * self.dpi.get() / 72.0
-        
+
         # problem - scale remembers last setting and font can become
         # enormous causing program to crash
         # save/restore prevents the problem
         ctx.set_font_size (size)
-        
+
         w, h = ctx.text_extents (s)[2:4]
         ctx.restore()
-            
+
         return w, h
 
-                              
+
     def new_gc(self):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        #gc = GraphicsContextCairo (renderer=self, surface=self.surface)
-        #return gc
+        self.ctx.restore()  # matches save() in set_ctx_from_surface()
+        self.ctx.save()
         return GraphicsContextCairo (renderer=self)
 
 
     def points_to_pixels(self, points):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
         return points/72.0 * self.dpi.get()
-     
+
 
 class GraphicsContextCairo(GraphicsContextBase):
     _joind = {
@@ -438,15 +428,13 @@ class GraphicsContextCairo(GraphicsContextBase):
         'round'      : cairo.LINE_CAP_ROUND,
         }
 
-    
-    #def __init__(self, renderer, surface):
+
     def __init__(self, renderer):
         GraphicsContextBase.__init__(self)
         self.renderer = renderer
-        #self.ctx = cairo.Context (surface)
         self.ctx = renderer.ctx
 
-        
+
     def set_alpha(self, alpha):
         self._alpha = alpha
         rgb = self._rgb
@@ -466,28 +454,18 @@ class GraphicsContextCairo(GraphicsContextBase):
 
 
     def set_clip_rectangle(self, rectangle):
-        # Cairo < 0.4.0: clipping is currently extremely slow
-        # Cairo 0.4.0  : pixel-aligned rectangular clip-regions are now faster
         self._cliprect = rectangle
 
         x,y,w,h = rectangle
-        # pixel-aligned clip-regions are faster        
+        # pixel-aligned clip-regions are faster
         x,y,w,h = round(x), round(y), round(w), round(h)
         ctx = self.ctx
         ctx.new_path()
         ctx.rectangle (x, self.renderer.height - h - y, w, h)
+        ctx.clip ()
+        # Alternative: just set _cliprect here and actually set cairo clip rect
+        # in fill_and_stroke() inside ctx.save() ... ctx.restore()
 
-        #ctx.save()     # uncomment to view the clip rectangle
-        #ctx.set_source_rgb(1,0,0)
-        #ctx.set_line_width(6)
-        #ctx.stroke()
-        #ctx.restore()        
-
-        #ctx.init_clip() # not needed? used unsuccessfully to fix clip problem
-        # when uncomment ctx.clip() it causes problems in line_styles.py
-        # - multiple axes, only the first one has its background drawn
-        #ctx.clip ()
-        
 
     def set_dashes(self, offset, dashes):
         self._dashes = offset, dashes
@@ -496,7 +474,7 @@ class GraphicsContextCairo(GraphicsContextBase):
         else:
             self.ctx.set_dash (
                self.renderer.points_to_pixels (numx.asarray(dashes)), offset)
-        
+
 
     def set_foreground(self, fg, isRGB=None):
         GraphicsContextBase.set_foreground(self, fg, isRGB)
@@ -506,7 +484,7 @@ class GraphicsContextCairo(GraphicsContextBase):
     def set_graylevel(self, frac):
         GraphicsContextBase.set_graylevel(self, frac)
         self.ctx.set_source_rgb(*self._rgb)
-        
+
 
     def set_joinstyle(self, js):
         if js in ('miter', 'round', 'bevel'):
@@ -520,68 +498,51 @@ class GraphicsContextCairo(GraphicsContextBase):
         self._linewidth = w
         self.ctx.set_line_width (self.renderer.points_to_pixels(w))
 
-        
+
 def new_figure_manager(num, *args, **kwargs): # called by backends/__init__.py
     """
     Create a new figure manager instance
     """
     if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-    thisFig = Figure(*args, **kwargs)
+    FigureClass = kwargs.pop('FigureClass', Figure)
+    thisFig = FigureClass(*args, **kwargs)
     canvas  = FigureCanvasCairo(thisFig)
     manager = FigureManagerBase(canvas, num)
     return manager
 
 
 class FigureCanvasCairo (FigureCanvasBase):
-    def print_figure(self, filename, dpi=150, facecolor='w', edgecolor='w',
-                     orientation='portrait'):
+    def print_figure(self, fo, dpi=150, facecolor='w', edgecolor='w',
+                     orientation='portrait', format=None, **kwargs):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
         # settings for printing
         self.figure.dpi.set(dpi)
         self.figure.set_facecolor(facecolor)
-        self.figure.set_edgecolor(edgecolor)        
+        self.figure.set_edgecolor(edgecolor)
 
-        # if isinstance(filename, file):   # eg when do savefig(sys.stdout)
-        #    _save_png (self.figure, filename) # assume PNG format
-        # else:
-        if True:
-            root, ext = os.path.splitext(filename)       
-            ext = ext[1:]
-            if ext == '':
-                ext      = IMAGE_FORMAT_DEFAULT
-                filename = filename + '.' + ext
+        if format is None and isinstance (fo, basestring):
+            # get format from filename extension
+            format = os.path.splitext(fo)[1][1:]
 
-            ext = ext.lower()
-            if ext in ('pdf', 'png', 'ps'):  # native formats
-                # try:
-                #    fileObject = file(filename,'wb')
-                # except IOError, exc:
-                #    warnings.warn("%s: %s" % (exc.filename, exc.strerror))
-                # else:
-            
-                # if ext == 'png': _save_png (fileObject)
-                if ext == 'png': self._save_png (filename)
-                # else:            _save_ps_pdf (self.figure, fileObject, ext,
-                #                               orientation)
-                else:    self._save_ps_pdf (self.figure, filename, ext, orientation)
-                # fileObject.close()
-            
-            elif ext in ('eps', 'svg'): # backend_svg/ps
-                if ext == 'svg':
-                    from backend_svg import FigureCanvasSVG as FigureCanvas
-                else:
-                    from backend_ps import FigureCanvasPS  as FigureCanvas
-                fc = FigureCanvas(self.figure)
-                fc.print_figure(filename, dpi, facecolor, edgecolor,
-                                orientation)
+        if format is not None:
+            format = format.lower()
 
-            else:
-                warnings.warn('Format "%s" is not supported.\n'
-                              'Supported formats: '
-                              '%s.' % (ext, ', '.join(IMAGE_FORMAT)))
+        if format == 'png':
+            self._save_png (fo)
+        elif format in ('pdf', 'ps', 'svg'):
+            self._save (fo, format, orientation, **kwargs)
+        elif format == 'eps': # backend_ps for eps
+            from backend_ps import FigureCanvasPS  as FigureCanvas
+            fc = FigureCanvas(self.figure)
+            fc.print_figure (filename, dpi, facecolor, edgecolor,
+                             orientation, **kwargs)
+        else:
+            warnings.warn('Format "%s" is not supported.\n'
+                          'Supported formats: '
+                          '%s.' % (format, ', '.join(IMAGE_FORMAT)))
 
-        
-    def _save_png (self, filename):
+
+    def _save_png (self, fobj):
         width, height = self.get_width_height()
 
         renderer = RendererCairo (self.figure.dpi)
@@ -590,32 +551,47 @@ class FigureCanvasCairo (FigureCanvasBase):
         renderer.set_ctx_from_surface (surface)
 
         self.figure.draw (renderer)
-        surface.write_to_png (filename)
-        
+        surface.write_to_png (fobj)
 
-    def _save_ps_pdf (self, figure, filename, ext, orientation):
-        # Cairo produces PostScript Level 3
-        # 'ggv' can't read cairo ps files, but 'gv' can
+
+    def _save (self, fo, format, orientation, **kwargs):
+        # save PDF/PS/SVG
+        orientation = kwargs.get('orientation', 'portrait')
 
         dpi = 72
-        figure.dpi.set (dpi)
-        w_in, h_in = figure.get_size_inches()
+        self.figure.dpi.set (dpi)
+        w_in, h_in = self.figure.get_size_inches()
         width_in_points, height_in_points = w_in * dpi, h_in * dpi
-    
+
         if orientation == 'landscape':
             width_in_points, height_in_points = (height_in_points,
                                                  width_in_points)
-        
-        if ext == 'ps':
-            surface = cairo.PSSurface (filename, width_in_points,
-                                       height_in_points)
-        else: # pdf
-            surface = cairo.PDFSurface (filename, width_in_points,
-                                        height_in_points)
+
+        if format == 'ps':
+            if not cairo.HAS_PS_SURFACE:
+                raise RuntimeError ('cairo has not been compiled with PS '
+                                    'support enabled')
+            surface = cairo.PSSurface (fo, width_in_points, height_in_points)
+        elif format == 'pdf':
+            if not cairo.HAS_PDF_SURFACE:
+                raise RuntimeError ('cairo has not been compiled with PDF '
+                                    'support enabled')
+            surface = cairo.PDFSurface (fo, width_in_points, height_in_points)
+        elif format == 'svg':
+            if not cairo.HAS_SVG_SURFACE:
+                raise RuntimeError ('cairo has not been compiled with SVG '
+                                    'support enabled')
+            surface = cairo.SVGSurface (fo, width_in_points, height_in_points)
+        else:
+           warnings.warn ("unknown format: %s" % format)
+           return
+
         # surface.set_dpi() can be used
-        # FIXME - use self._renderer?
-        renderer._set_ctx_from_surface (surface)
-    
+        renderer = RendererCairo (self.figure.dpi)
+        renderer.set_width_height (width_in_points, height_in_points)
+        renderer.set_ctx_from_surface (surface)
+        ctx = renderer.ctx
+
         if orientation == 'landscape':
             ctx.rotate (numx.pi/2)
             ctx.translate (0, -height_in_points)
@@ -626,10 +602,7 @@ class FigureCanvasCairo (FigureCanvasBase):
             # TODO:
             # add portrait/landscape checkbox to FileChooser
 
-        renderer = RendererCairo (figure.dpi)
-        renderer._set_width_height (width_in_points, height_in_points)
-        renderer.surface = surface
-        figure.draw (renderer)
+        self.figure.draw (renderer)
 
         show_fig_border = False  # for testing figure orientation and scaling
         if show_fig_border:
@@ -642,5 +615,6 @@ class FigureCanvasCairo (FigureCanvasBase):
             ctx.select_font_face ('sans-serif')
             ctx.set_font_size(20)
             ctx.show_text('Origin corner')
-    
+
         ctx.show_page()
+        surface.finish()
