@@ -6,12 +6,12 @@ import os
 
 import matplotlib
 from matplotlib.figure import Figure
-from backend_agg import FigureCanvasAgg
-from backend_gtk import gtk, FigureManagerGTK, FigureCanvasGTK,\
+from matplotlib.backends.backend_agg import FigureCanvasAgg
+from matplotlib.backends.backend_gtk import gtk, FigureManagerGTK, FigureCanvasGTK,\
      show, draw_if_interactive,\
      error_msg_gtk, NavigationToolbar, PIXELS_PER_INCH, backend_version, \
      NavigationToolbar2GTK
-from _gtkagg import agg_to_gtk_drawable
+from matplotlib.backends._gtkagg import agg_to_gtk_drawable
 
 
 DEBUG = False
@@ -38,14 +38,16 @@ def new_figure_manager(num, *args, **kwargs):
     Create a new figure manager instance
     """
     if DEBUG: print 'backend_gtkagg.new_figure_manager'
-    thisFig = Figure(*args, **kwargs)
+    FigureClass = kwargs.pop('FigureClass', Figure)
+    thisFig = FigureClass(*args, **kwargs)
     canvas = FigureCanvasGTKAgg(thisFig)
     return FigureManagerGTKAgg(canvas, num)
-
+    if DEBUG: print 'backend_gtkagg.new_figure_manager done'
 
 class FigureCanvasGTKAgg(FigureCanvasGTK, FigureCanvasAgg):
 
     def configure_event(self, widget, event=None):
+
         if DEBUG: print 'FigureCanvasGTKAgg.configure_event'
         if widget.window is None:
             return 
@@ -60,38 +62,72 @@ class FigureCanvasGTKAgg(FigureCanvasGTK, FigureCanvasAgg):
         dpival = self.figure.dpi.get()
         winch = w/dpival
         hinch = h/dpival
-        self.figure.set_figsize_inches(winch, hinch)
+        self.figure.set_size_inches(winch, hinch)
         self._need_redraw = True
-        
+        self.resize_event()
         if DEBUG: print 'FigureCanvasGTKAgg.configure_event end'        
         return True
     
     def _render_figure(self, pixmap, width, height):
+        if DEBUG: print 'FigureCanvasGTKAgg.render_figure'
         FigureCanvasAgg.draw(self)
-        agg_to_gtk_drawable(pixmap, self.renderer._renderer, None)
+        if DEBUG: print 'FigureCanvasGTKAgg.render_figure pixmap', pixmap
+        #agg_to_gtk_drawable(pixmap, self.renderer._renderer, None)
+
+        buf = self.buffer_rgba(0,0)
+        ren = self.get_renderer()
+        w = int(ren.width)
+        h = int(ren.height)
+        pixbuf = gtk.gdk.pixbuf_new_from_data(
+            buf, gtk.gdk.COLORSPACE_RGB,  True, 8, w, h, w*4)
+        pixmap.draw_pixbuf(pixmap.new_gc(), pixbuf, 0, 0, 0, 0, w, h,
+                           gtk.gdk.RGB_DITHER_NONE, 0, 0)
+        if DEBUG: print 'FigureCanvasGTKAgg.render_figure done'
 
     def blit(self, bbox=None):
+        if DEBUG: print 'FigureCanvasGTKAgg.blit'
+        if DEBUG: print 'FigureCanvasGTKAgg.blit', self._pixmap
         agg_to_gtk_drawable(self._pixmap, self.renderer._renderer, bbox)
-        x, y, w, h = self.allocation
-        self.window.draw_drawable (self.style.fg_gc[self.state], self._pixmap,
-                                   x, y, x, y, w, h)
 
-    def print_figure(self, filename, dpi=150,
-                     facecolor='w', edgecolor='w',
-                     orientation='portrait'):
+        x, y, w, h = self.allocation
+
+        self.window.draw_drawable (self.style.fg_gc[self.state], self._pixmap,
+                                   0, 0, 0, 0, w, h)
+        if DEBUG: print 'FigureCanvasGTKAgg.done'
+
+    def print_figure(self, filename, dpi=None, facecolor='w', edgecolor='w',
+                     orientation='portrait', **kwargs):
         if DEBUG: print 'FigureCanvasGTKAgg.print_figure'
         # delete the renderer to prevent improper blitting after print
 
+        if dpi is None: dpi = matplotlib.rcParams['savefig.dpi']
         root, ext = os.path.splitext(filename)       
         ext = ext.lower()[1:]
         if ext == 'jpg':
-            FigureCanvasGTK.print_figure(self, filename, dpi, facecolor,
-                                         edgecolor, orientation)
+            FigureCanvasGTK.print_figure(self, filename, dpi, facecolor, 
+                                         edgecolor, orientation, **kwargs)
             
         else:
             agg = self.switch_backends(FigureCanvasAgg)
-            try: agg.print_figure(filename, dpi, facecolor, edgecolor, orientation)
+            try:
+                agg.print_figure(filename, dpi, facecolor, edgecolor,
+                                 orientation, **kwargs)
             except IOError, msg:
                 error_msg_gtk('Failed to save\nError message: %s'%(msg,), self)
+            except:
+                self.figure.set_canvas(self)
+                raise
 
         self.figure.set_canvas(self)
+        if DEBUG: print 'FigureCanvasGTKAgg.print_figure done'
+
+
+
+"""\
+Traceback (most recent call last):
+  File "/home/titan/johnh/local/lib/python2.3/site-packages/matplotlib/backends/backend_gtk.py", line 304, in expose_event
+    self._render_figure(self._pixmap, w, h)
+  File "/home/titan/johnh/local/lib/python2.3/site-packages/matplotlib/backends/backend_gtkagg.py", line 77, in _render_figure
+    pixbuf = gtk.gdk.pixbuf_new_from_data(
+ValueError: data length (3156672) is less then required by the other parameters (3160608)
+"""
