@@ -92,7 +92,7 @@ Examples which work on this release:
  (3) - Clipping seems to be broken.
 """
 
-cvs_id = '$Id: backend_wx.py,v 1.30 2005/06/29 12:57:13 jdh2358 Exp $'
+cvs_id = '$Id: backend_wx.py 3011 2007-02-12 23:10:43Z jdh2358 $'
 
 import sys, os, os.path, math, StringIO
 
@@ -275,7 +275,7 @@ class RendererWx(RendererBase):
         return self.width, self.height
 
 
-    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2):
+    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2, rotation):
         """
         Draw an arc centered at x,y with width and height and angles
         from 0.0 to 360.0.
@@ -890,7 +890,7 @@ The current aspect ration will be kept."""
         self.gui_repaint()
 
 
-    def draw(self):
+    def draw(self, repaint=True):
         """
         Render the figure using RendererWx instance renderer, or using a
         previously defined renderer if none is specified.
@@ -898,7 +898,8 @@ The current aspect ration will be kept."""
         DEBUG_MSG("draw()", 1, self)
         self.renderer = RendererWx(self.bitmap, self.figure.dpi)
         self.figure.draw(self.renderer)
-        self.gui_repaint()
+        if repaint:
+            self.gui_repaint()
 
     def _get_imagesave_wildcards(self):
         'return the wildcard string for the filesave dialog'
@@ -911,27 +912,26 @@ The current aspect ration will be kept."""
                "PNG (*.png)|*.png|"  \
                "XPM (*.xpm)|*.xpm"
 
-    def gui_repaint(self):
+    def gui_repaint(self, drawDC=None):
         """
-        Performs update of the displayed image on the GUI canvas
-
-        MUST NOT be called during a Paint event
+        Performs update of the displayed image on the GUI canvas, using the
+        supplied device context.  If drawDC is None, a ClientDC will be used to
+        redraw the image.
         """
         DEBUG_MSG("gui_repaint()", 1, self)
-        drawDC=wx.ClientDC(self)
+        if drawDC is None:
+            drawDC=wx.ClientDC(self)
+
         drawDC.BeginDrawing()
-        #drawDC.Clear()
         drawDC.DrawBitmap(self.bitmap, 0, 0)
         drawDC.EndDrawing()
 
-    def print_figure(self, filename, dpi=150,
-                     facecolor='w', edgecolor='w',
-                     orientation='portrait'):
-
+    def print_figure(self, filename, dpi=None, facecolor='w', edgecolor='w',
+                     orientation='portrait', **kwargs):
         """
         Render the figure to hardcopy
         """
-
+        if dpi is None: dpi = matplotlib.rcParams['savefig.dpi']
         DEBUG_MSG("print_figure()", 1, self)
         # Save state information, and set save DPI
 
@@ -947,7 +947,8 @@ The current aspect ration will be kept."""
             ps = self.switch_backends(FigureCanvasPS)
             ps.figure.dpi.set(72)
 
-            ps.print_figure(filename, 72, facecolor, edgecolor)
+            ps.print_figure(filename, 72, facecolor, edgecolor, orientation, 
+                            **kwargs)
             self.figure.dpi.set(origDPI)
             self.figure.set_canvas(self)
             return
@@ -960,7 +961,8 @@ The current aspect ration will be kept."""
             origDPI = self.figure.dpi.get()
             svg = self.switch_backends(FigureCanvasSVG)
             svg.figure.dpi.set(72)
-            svg.print_figure(filename, 72, facecolor, edgecolor)
+            svg.print_figure(filename, 72, facecolor, edgecolor, orientation,
+                             **kwargs)
             self.figure.dpi.set(origDPI)
             self.figure.set_canvas(self)
             return
@@ -1043,27 +1045,9 @@ The current aspect ration will be kept."""
         if not self._isRealized:
             self.realize()
         # Render to the bitmap
-        self.draw()
-        # Must use wxPaintDC during paint event
-
-        l,b,w,h = self.figure.bbox.get_bounds()
-        w = int(math.ceil(w))
-        h = int(math.ceil(h))
-
-
-        # I decoupled this from GraphicsContextWx so it would play
-        # nice with wxagg
-        memDC =wx.MemoryDC()
-        memDC.SelectObject(self.bitmap)
-        memDC.SetPen(wx.Pen('BLACK', 1, wx.SOLID))
-
-        drawDC=wx.PaintDC(self)
-
-        drawDC.BeginDrawing()
-        drawDC.Clear()
-        drawDC.Blit(0, 0, w, h, memDC, 0, 0)
-        #drawDC.DrawBitmap(self.bitmap, 0, 0)
-        drawDC.EndDrawing()
+        self.draw(repaint=False)
+        # Update the display using a PaintDC
+        self.gui_repaint(drawDC=wx.PaintDC(self))
         evt.Skip()
 
     def _onSize(self, evt):
@@ -1088,7 +1072,7 @@ The current aspect ration will be kept."""
         dpival = self.figure.dpi.get()
         winch = self._width/dpival
         hinch = self._height/dpival
-        self.figure.set_figsize_inches(winch, hinch)
+        self.figure.set_size_inches(winch, hinch)
 
         if self._isRealized:
             self.draw()
@@ -1226,7 +1210,8 @@ def new_figure_manager(num, *args, **kwargs):
         wxapp = wx.PySimpleApp()
         wxapp.SetExitOnFrameDelete(True)
 
-    fig = Figure(*args, **kwargs)
+    FigureClass = kwargs.pop('FigureClass', Figure)
+    fig = FigureClass(*args, **kwargs)
     frame = FigureFrameWx(num, fig)
     figmgr = frame.get_figure_manager()
     if matplotlib.is_interactive():
@@ -1339,8 +1324,9 @@ class FigureManagerWx(FigureManagerBase):
         FigureManagerBase.__init__(self, canvas, num)
         self.frame = frame
         self.window = frame
-        self.tb = frame.GetToolBar()
 
+        self.tb = frame.GetToolBar()
+        self.toolbar = self.tb  # consistent with other backends
         def notify_axes_change(fig):
             'this will be called whenever the current axes is changed'
             if self.tb != None: self.tb.update()
@@ -1386,7 +1372,7 @@ def _load_bitmap(filename):
     Returns a wx.Bitmap object
     """
 
-    basedir = rcParams['datapath']
+    basedir = os.path.join(rcParams['datapath'],'images')
 
     bmpFilename = os.path.normpath(os.path.join(basedir, filename))
     if not os.path.exists(bmpFilename):
