@@ -4,10 +4,86 @@ from the Python Cookbook -- hence the name cbook
 """
 from __future__ import generators
 import re, os, errno, sys, StringIO, traceback
+import matplotlib.numerix as nx
 
+try: set
+except NameError:
+    from sets import Set as set
 
 major, minor1, minor2, s, tmp = sys.version_info
 
+class CallbackRegistry:
+    """
+    Handle registering and disconnecting for a set of signals and
+    callbacks
+
+       signals = 'eat', 'drink', 'be merry'
+
+       def oneat(x):
+           print 'eat', x
+
+       def ondrink(x):
+           print 'drink', x
+
+       callbacks = CallbackRegistry(signals)
+
+       ideat = callbacks.connect('eat', oneat)
+       iddrink = callbacks.connect('drink', ondrink)
+
+       #tmp = callbacks.connect('drunk', ondrink) # this will raise a ValueError
+
+       callbacks.process('drink', 123)    # will call oneat
+       callbacks.process('eat', 456)      # will call ondrink
+       callbacks.process('be merry', 456) # nothing will be called
+       callbacks.disconnect(ideat)        # disconnect oneat
+       callbacks.process('eat', 456)      # nothing will be called
+
+    """
+    def __init__(self, signals):
+        'signals is a sequence of valid signals'
+        self.signals = set(signals)
+        # callbacks is a dict mapping the signal to a dictionary
+        # mapping callback id to the callback function
+        self.callbacks = dict([(s, dict()) for s in signals])
+        self._cid = 0
+
+    def _check_signal(self, s):
+        'make sure s is a valid signal or raise a ValueError'
+        if s not in self.signals:
+            signals = list(self.signals)
+            signals.sort()
+            raise ValueError('Unknown signal "%s"; valid signals are %s'%(s, signals))
+
+    def connect(self, s, func):
+        """
+        register func to be called when a signal s is generated
+        func will be called with args and kwargs
+        """
+        self._check_signal(s)
+        self._cid +=1
+        self.callbacks[s][self._cid] = func
+        return self._cid
+
+    def disconnect(self, cid):
+        """
+        disconnect the callback registered with callback id cid
+        """
+        for eventname, callbackd in self.callbacks.items():
+            try: del callbackd[cid]
+            except KeyError: continue
+            else: return
+
+    def process(self, s, *args, **kwargs):
+        """
+        process signal s.  All of the functions registered to receive
+        callbacks on s will be called with *args and **kwargs
+        """
+        self._check_signal(s)
+        for func in self.callbacks[s].values():
+            func(*args, **kwargs)
+
+
+
 class silent_list(list):
     """
     override repr when returning a list of matplotlib artists to
@@ -17,7 +93,7 @@ class silent_list(list):
     def __init__(self, type, seq=None):
         self.type = type
         if seq is not None: self.extend(seq)
-        
+
     def __repr__(self):
         return '<a list of %d %s objects>' % (len(self), self.type)
 
@@ -46,7 +122,8 @@ class Bunch:
    """
    def __init__(self, **kwds):
       self.__dict__.update(kwds)
-
+      
+      
 def unique(x):
    'Return a list of unique elements of x'
    return dict([ (val, 1) for val in x]).keys()
@@ -55,7 +132,7 @@ def iterable(obj):
     try: len(obj)
     except: return 0
     return 1
-      
+
 
 def is_string_like(obj):
     if hasattr(obj, 'shape'): return 0 # this is a workaround
@@ -75,6 +152,11 @@ def is_file_like(obj):
 def is_scalar(obj):
     return is_string_like(obj) or not iterable(obj)
 
+def is_numlike(obj):
+    try: obj+1
+    except TypeError: return False
+    else: return True
+
 def flatten(seq, scalarp=is_scalar):
     """
     this generator flattens nested containers such as
@@ -114,7 +196,7 @@ class Sorter:
    sort(list)       # default sort
    sort(list, 1)    # sort by index 1
    sort(dict, 'a')  # sort a list of dicts by key 'a'
-   
+
    """
 
    def _helper(self, data, aux, inplace):
@@ -228,8 +310,8 @@ class Null:
 def mkdirs(newdir, mode=0777):
    try: os.makedirs(newdir, mode)
    except OSError, err:
-      # Reraise the error unless it's about an already existing directory 
-      if err.errno != errno.EEXIST or not os.path.isdir(newdir): 
+      # Reraise the error unless it's about an already existing directory
+      if err.errno != errno.EEXIST or not os.path.isdir(newdir):
          raise
 
 
@@ -238,7 +320,7 @@ def dict_delall(d, keys):
     for key in keys:
         try: del d[key]
         except KeyError: pass
-        
+
 
 class RingBuffer:
     """ class that implements a not-yet-full buffer """
@@ -272,7 +354,7 @@ class RingBuffer:
        return self.data[i % len(self.data)]
 
 
-# use enumerate builtin if available, else use python version 
+# use enumerate builtin if available, else use python version
 try:
     import __builtin__
     enumerate = __builtin__.enumerate
@@ -285,7 +367,7 @@ except:
             yield i, seq[i]
 
 
-# use itertools.izip if available, else use python version 
+# use itertools.izip if available, else use python version
 try:
     import itertools
     izip = itertools.izip
@@ -298,21 +380,21 @@ except:
         while iterables:
             result = [i.next() for i in iterables]
             yield tuple(result)
-             
+
 
 def get_split_ind(seq, N):
    """seq is a list of words.  Return the index into seq such that
    len(' '.join(seq[:ind])<=N
    """
-  
+
    sLen = 0
    # todo: use Alex's xrange pattern from the cbook for efficiency
    for (word, ind) in zip(seq, range(len(seq))):
       sLen += len(word) + 1  # +1 to account for the len(' ')
       if sLen>=N: return ind
    return len(seq)
-     
-  
+
+
 def wrap(prefix, text, cols):
     'wrap text with prefix at length cols'
     pad = ' '*len(prefix.expandtabs())
@@ -333,6 +415,29 @@ def wrap(prefix, text, cols):
         ret += pad + ' '.join(line) + '\n'
     return ret
 
+def dedent(s):
+    """
+    Remove excess indentation from docstrings.
+
+    Discards any leading blank lines, then removes up to
+    n whitespace characters from each line, where n is
+    the number of leading whitespace characters in the
+    first line. It differs from textwrap.dedent in its
+    deletion of leading blank lines and its use of the
+    first non-blank line to determine the indentation.
+    """
+    if not s:      # includes case of s is None
+        return ''
+    lines = s.splitlines(True)
+    ii = 0
+    while lines[ii].strip() == '':
+        ii += 1
+    lines = lines[ii:]
+    nshift = len(lines[0]) - len(lines[0].lstrip())
+    for i, line in enumerate(lines):
+        nwhite = len(line) - len(line.lstrip())
+        lines[i] = line[min(nshift, nwhite):]
+    return ''.join(lines)
 
 
 
@@ -372,7 +477,7 @@ def get_recursive_filelist(args):
     return the files as a list of strings
     """
     files = []
-    
+
     for arg in args:
         if os.path.isfile(arg):
             files.append(arg)
@@ -433,14 +538,14 @@ def allpairs(x):
     return all possible pairs in sequence x
 
     Condensed by Alex Martelli from this thread on c.l.python
-    http://groups.google.com/groups?q=all+pairs+group:*python*&hl=en&lr=&ie=UTF-8&selm=mailman.4028.1096403649.5135.python-list%40python.org&rnum=1    
+    http://groups.google.com/groups?q=all+pairs+group:*python*&hl=en&lr=&ie=UTF-8&selm=mailman.4028.1096403649.5135.python-list%40python.org&rnum=1
     """
     return [ (s, f) for i, f in enumerate(x) for s in x[i+1:] ]
 
-        
 
 
-# python 2.2 dicts don't have pop
+
+# python 2.2 dicts don't have pop--but we don't support 2.2 any more
 def popd(d, *args):
     """
     Should behave like python2.3 pop method; d is a dict
@@ -451,7 +556,7 @@ def popd(d, *args):
 
     # returns value for key if key exists, else default.  Delete key,
     # val item if it exists.  Will not raise a KeyError
-    val = popd(d, key, default)      
+    val = popd(d, key, default)
     """
     if len(args)==1:
         key = args[0]
@@ -465,19 +570,36 @@ def popd(d, *args):
     return val
 
 
+class maxdict(dict):
+    """
+    A dictionary with a maximum size; this doesn't override all the
+    relevant methods to contrain size, just setitem, so use with
+    caution
+    """
+    def __init__(self, maxsize):
+        dict.__init__(self)
+        self.maxsize = maxsize
+        self._killkeys = []
+    def __setitem__(self, k, v):
+        if len(self)>=self.maxsize:
+            del self[self._killkeys[0]]
+            del self._killkeys[0]
+        dict.__setitem__(self, k, v)
+        self._killkeys.append(k)
+
 
 
 class Stack:
     """
     Implement a stack where elements can be pushed on and you can move
-    back and forth.  But no pop.  Should mimib home / back / forward
+    back and forth.  But no pop.  Should mimic home / back / forward
     in a browser
     """
 
     def __init__(self, default=None):
         self.clear()
         self._default = default
-    
+
     def __call__(self):
         'return the current element, or None'
         if not len(self._elements): return self._default
@@ -503,7 +625,7 @@ class Stack:
         self._elements.append(o)
         self._pos = len(self._elements)-1
         return self()
-    
+
     def home(self):
         'push the first element onto the top of the stack'
         if not len(self._elements): return
@@ -519,9 +641,13 @@ class Stack:
         self._elements = []
 
     def bubble(self, o):
-        'raise o to the top of the stack and return o.  o must b in the stack'
+        """
+        raise o to the top of the stack and return o.  o must be in
+        the stack
+        """
 
-        if o not in self._elements: raise ValueError('Unknown element o')
+        if o not in self._elements:
+            raise ValueError('Unknown element o')
         old = self._elements[:]
         self.clear()
         bubbles = []
@@ -534,13 +660,14 @@ class Stack:
 
     def remove(self, o):
         'remove element o from the stack'
-        if o not in self._elements: raise ValueError('Unknown element o')
+        if o not in self._elements:
+            raise ValueError('Unknown element o')
         old = self._elements[:]
         self.clear()
         for thiso in old:
             if thiso==o: continue
             else: self.push(thiso)
-        
+
 def popall(seq):
     'empty a list'
     for i in xrange(len(seq)): seq.pop()
@@ -548,7 +675,7 @@ def popall(seq):
 def finddir(o, match, case=False):
     """
     return all attributes of o which match string in match.  if case
-    is True require an exact case match.  
+    is True require an exact case match.
     """
     if case:
         names = [(name,name) for name in dir(o) if is_string_like(name)]
@@ -562,9 +689,74 @@ def reverse_dict(d):
     return dict([(v,k) for k,v in d.items()])
 
 
+def report_memory(i=0):  # argument may go away
+    'return the memory consumed by process'
+    pid = os.getpid()
+    if sys.platform=='sunos5':
+        a2 = os.popen('ps -p %d -o osz' % pid).readlines()
+        mem = int(a2[-1].strip())
+    elif sys.platform.startswith('linux'):
+        a2 = os.popen('ps -p %d -o rss,sz' % pid).readlines()
+        mem = int(a2[1].split()[1])
+    elif sys.platform.startswith('darwin'):
+        a2 = os.popen('ps -p %d -o rss,vsz' % pid).readlines()
+        mem = int(a2[1].split()[0])
+
+    return mem
+
+class MemoryMonitor:
+    def __init__(self, nmax=20000):
+        self._nmax = nmax
+        self._mem = nx.zeros((self._nmax,), nx.Int32)
+        self.clear()
+
+    def clear(self):
+        self._n = 0
+        self._overflow = False
+
+    def __call__(self):
+        mem = report_memory()
+        if self._n < self._nmax:
+            self._mem[self._n] = mem
+            self._n += 1
+        else:
+            self._overflow = True
+        return mem
+
+    def report(self, segments=4):
+        n = self._n
+        segments = min(n, segments)
+        dn = int(n/segments)
+        ii = range(0, n, dn)
+        ii[-1] = n-1
+        print
+        print 'memory report: i, mem, dmem, dmem/nloops'
+        print 0, self._mem[0]
+        for i in range(1, len(ii)):
+            di = ii[i] - ii[i-1]
+            if di == 0:
+                continue
+            dm = self._mem[ii[i]] - self._mem[ii[i-1]]
+            print '%5d %5d %3d %8.3f' % (ii[i], self._mem[ii[i]],
+                                            dm, dm / float(di))
+        if self._overflow:
+            print "Warning: array size was too small for the number of calls."
+
+    def xy(self, i0=0, isub=1):
+        x = nx.arange(i0, self._n, isub)
+        return x, self._mem[i0:self._n:isub]
+
+    def plot(self, i0=0, isub=1):
+        from pylab import figure, show
+        fig = figure()
+        ax = fig.add_subplot(111)
+        ax.plot(*self.xy(i0, isub))
+        show()
+
+
 if __name__=='__main__':
     assert( allequal([1,1,1]) )
     assert(not  allequal([1,1,0]) )
     assert( allequal([]) )
     assert( allequal(('a', 'a')))
-    assert( not allequal(('a', 'b')))    
+    assert( not allequal(('a', 'b')))
