@@ -1,20 +1,27 @@
+"""
+Figure class -- add docstring here!
+"""
 import sys
+import artist
 from artist import Artist
 from axes import Axes, Subplot, PolarSubplot, PolarAxes
-from cbook import flatten, allequal, popd, Stack, iterable
+from cbook import flatten, allequal, Stack, iterable, dedent
 import _image
-from colors import normalize
+import colorbar as cbar
+from colors import Normalize, rgb2hex
 from image import FigureImage
 from matplotlib import rcParams
-from patches import Rectangle
+from patches import Rectangle, Polygon
 from text import Text, _process_text_args
 
 from legend import Legend
 from transforms import Bbox, Value, Point, get_bbox_transform, unit_bbox
-from numerix import array, clip, transpose
-from mlab import linspace
+from numerix import array, clip, transpose, minimum, maximum
+from mlab import linspace, meshgrid
 from ticker import FormatStrFormatter
-
+from cm import ScalarMappable
+from contour import ContourSet
+import warnings
 
 class SubplotParams:
     """
@@ -101,19 +108,15 @@ class Figure(Artist):
                  facecolor = None,  # defaults to rc figure.facecolor
                  edgecolor = None,  # defaults to rc figure.edgecolor
                  linewidth = 1.0,   # the default linewidth of the frame
-                 frameon = True,
+                 frameon = True,    # whether or not to draw the figure frame
                  subplotpars = None, # default to rc
                  ):
         """
-        paper size is a w,h tuple in inches
-        DPI is dots per inch
+        figsize is a w,h tuple in inches
+        dpi is dots per inch
         subplotpars is a SubplotParams instance, defaults to rc
         """
         Artist.__init__(self)
-        #self.set_figure(self)
-        self._axstack = Stack()  # maintain the current axes
-        self._axobservers = []
-        self._seen = {}          # axes args we've seen
 
         if figsize is None  : figsize   = rcParams['figure.figsize']
         if dpi is None      : dpi       = rcParams['figure.dpi']
@@ -127,6 +130,7 @@ class Figure(Artist):
         self.ur = Point( self.figwidth*self.dpi,
                          self.figheight*self.dpi )
         self.bbox = Bbox(self.ll, self.ur)
+
         self.frameon = frameon
 
         self.transFigure = get_bbox_transform( unit_bbox(), self.bbox)
@@ -148,10 +152,58 @@ class Figure(Artist):
 
         self.subplotpars = subplotpars
 
+        self._axstack = Stack()  # maintain the current axes
+        self.axes = []
         self.clf()
 
         self._cachedRenderer = None
 
+    def autofmt_xdate(self, bottom=0.2, rotation=30, ha='right'):
+        """
+        A common use case is a number of subplots with shared xaxes
+        where the x-axis is date data.  The ticklabels are often
+        long,and it helps to rotate them on the bottom subplot and
+        turn them off on other subplots.  This function will raise a
+        RuntimeError if any of the Axes are not Subplots.
+
+        bottom : the bottom of the subplots for subplots_adjust
+        rotation: the rotation of the xtick labels
+        ha : the horizontal alignment of the xticklabels
+
+        
+        """
+
+        for ax in self.get_axes():
+            if not hasattr(ax, 'is_last_row'):
+                raise RuntimeError('Axes must be subplot instances; found %s'%type(ax))
+            if ax.is_last_row():
+                for label in ax.get_xticklabels():
+                    label.set_ha(ha)
+                    label.set_rotation(rotation)                
+            else:
+                for label in ax.get_xticklabels():
+                    label.set_visible(False)
+        self.subplots_adjust(bottom=bottom)
+        
+    def get_children(self):
+        'get a list of artists contained in the figure'
+        children = [self.figurePatch]
+        children.extend(self.axes)
+        children.extend(self.lines)
+        children.extend(self.patches)
+        children.extend(self.texts)
+        children.extend(self.images)
+        children.extend(self.legends)
+        return children
+
+    def pick(self, mouseevent):
+        for a in self.get_children():
+            a.pick(mouseevent)
+
+    def get_window_extent(self, *args, **kwargs):
+        'get the figure bounding box in display space; kwargs are void'
+        return self.bbox
+
     def set_canvas(self, canvas):
         """
         Set the canvas the contains the figure
@@ -203,7 +255,7 @@ class Figure(Artist):
           * cmap is a cm colormap instance, eg cm.jet.  If None, default to
             the rc image.cmap valuex
 
-          * norm is a matplotlib.colors.normalize instance; default is
+          * norm is a matplotlib.colors.Normalize instance; default is
             normalization().  This scales luminance -> 0-1
 
           * vmin and vmax are used to scale a luminance image to 0-1.  If
@@ -234,16 +286,30 @@ class Figure(Artist):
         self.images.append(im)
         return im
 
+    def set_figsize_inches(self, *args, **kwargs):
+        import warnings
+        warnings.warn('Use set_size_inches instead!', DeprecationWarning)
+        self.set_size_inches(*args, **kwargs)
 
-    def set_figsize_inches(self, *args):
+    def set_size_inches(self, *args, **kwargs):
         """
+        set_size_inches(w,h, forward=False)
+
         Set the figure size in inches
 
-        Usage: set_figsize_inches(self, w,h)  OR
-               set_figsize_inches(self, (w,h) )
+        Usage: set_size_inches(self, w,h)  OR
+               set_size_inches(self, (w,h) )
+
+        optional kwarg forward=True will cause the canvas size to be
+        automatically updated; eg you can resize the figure window
+        from the shell
+
+        WARNING: forward=True is broken on all backends except GTK*
 
         ACCEPTS: a w,h tuple with w,h in inches
         """
+
+        forward = kwargs.get('forward', False)
         if len(args)==1:
             w,h = args[0]
         else:
@@ -251,6 +317,14 @@ class Figure(Artist):
         self.figwidth.set(w)
         self.figheight.set(h)
 
+        if forward:
+            dpival = self.dpi.get()
+            canvasw = w*dpival
+            canvash = h*dpival
+            manager = getattr(self.canvas, 'manager', None)
+            if manager is not None:
+                manager.resize(int(canvasw), int(canvash))
+
     def get_size_inches(self):
         return self.figwidth.get(), self.figheight.get()
 
@@ -342,7 +416,7 @@ class Figure(Artist):
 
         def fixitems(items):
             #items may have arrays and lists in them, so convert them
-            # to tuples for the kyey
+            # to tuples for the key
             ret = []
             for k, v in items:
                 if iterable(v): v = tuple(v)
@@ -363,11 +437,12 @@ class Figure(Artist):
         """
         Add an a axes with axes rect [left, bottom, width, height] where all
         quantities are in fractions of figure width and height.  kwargs are
-        legal Axes kwargs plus"polar" which sets whether to create a polar axes
+        legal Axes kwargs plus "polar" which sets whether to create a polar axes
 
-            add_axes((l,b,w,h))
-            add_axes((l,b,w,h), frameon=False, axisbg='g')
-            add_axes((l,b,w,h), polar=True)
+            rect = l,b,w,h
+            add_axes(rect)
+            add_axes(rect, frameon=False, axisbg='g')
+            add_axes(rect, polar=True)
             add_axes(ax)   # add an Axes instance
 
 
@@ -376,13 +451,16 @@ class Figure(Artist):
         behavior, eg you want to force the creation of a new axes, you must
         use a unique set of args and kwargs.  The artist "label" attribute has
         been exposed for this purpose.  Eg, if you want two axes that are
-        otherwise identical to be added to the axes, make sure you give them
+        otherwise identical to be added to the figure, make sure you give them
         unique labels:
 
-            add_axes((l,b,w,h), label='1')
-            add_axes((l,b,w,h), label='2')
+            add_axes(rect, label='axes1')
+            add_axes(rect, label='axes2')
 
         The Axes instance will be returned
+
+        The following kwargs are supported:
+        %(Axes)s
         """
 
         key = self._make_key(*args, **kwargs)
@@ -395,10 +473,10 @@ class Figure(Artist):
         if not len(args): return
         if isinstance(args[0], Axes):
             a = args[0]
-            a.set_figure(self)
+            assert(a.get_figure() is self)
         else:
             rect = args[0]
-            ispolar = popd(kwargs, 'polar', False)
+            ispolar = kwargs.pop('polar', False)
 
             if ispolar:
                 a = PolarAxes(self, rect, **kwargs)
@@ -412,6 +490,8 @@ class Figure(Artist):
         self._seen[key] = a
         return a
 
+    add_axes.__doc__ = dedent(add_axes.__doc__) % artist.kwdocd
+
     def add_subplot(self, *args, **kwargs):
         """
         Add a subplot.  Examples
@@ -426,10 +506,12 @@ class Figure(Artist):
 
         If the figure already has a subplot with key *args, *kwargs then it will
         simply make that subplot current and return it
+
+        The following kwargs are supported:
+        %(Axes)s
         """
 
         key = self._make_key(*args, **kwargs)
-
         if self._seen.has_key(key):
             ax = self._seen[key]
             self.sca(ax)
@@ -438,11 +520,11 @@ class Figure(Artist):
 
         if not len(args): return
 
-        if isinstance(args[0], Subplot) or isinstance(args, PolarSubplot):
+        if isinstance(args[0], Subplot) or isinstance(args[0], PolarSubplot):
             a = args[0]
-            a.set_figure(self)
+            assert(a.get_figure() is self)
         else:
-            ispolar = popd(kwargs, 'polar', False)
+            ispolar = kwargs.pop('polar', False)
             if ispolar:
                 a = PolarSubplot(self, *args, **kwargs)
             else:
@@ -454,12 +536,19 @@ class Figure(Artist):
         self.sca(a)
         self._seen[key] = a
         return a
+    add_subplot.__doc__ = dedent(add_subplot.__doc__) % artist.kwdocd
 
     def clf(self):
         """
         Clear the figure
         """
-        self.axes = []
+        for ax in tuple(self.axes):  # Iterate over the copy.
+            ax.cla()
+            self.delaxes(ax)         # removes ax from self.axes
+
+        toolbar = getattr(self.canvas, 'toolbar', None)
+        if toolbar is not None:
+            toolbar.update()
         self._axstack.clear()
         self._seen = {}
         self.lines = []
@@ -467,6 +556,7 @@ class Figure(Artist):
         self.texts=[]
         self.images = []
         self.legends = []
+        self._axobservers = []
 
     def clear(self):
         """
@@ -476,35 +566,35 @@ class Figure(Artist):
 
     def draw(self, renderer):
         """
-        Render the figure using RendererGD instance renderer
+        Render the figure using Renderer instance renderer
         """
         # draw the figure bounding box, perhaps none for white figure
         #print 'figure draw'
         if not self.get_visible(): return
         renderer.open_group('figure')
         self.transFigure.freeze()  # eval the lazy objects
+
         if self.frameon: self.figurePatch.draw(renderer)
 
         for p in self.patches: p.draw(renderer)
         for l in self.lines: l.draw(renderer)
 
-        if len(self.images)==1:
-            im = self.images[0]
-            im.draw(renderer)
-        elif len(self.images)>1:
+        if len(self.images)<=1 or renderer.option_image_nocomposite() or not allequal([im.origin for im in self.images]):
+            for im in self.images:
+                im.draw(renderer)
+        else:
             # make a composite image blending alpha
             # list of (_image.Image, ox, oy)
-            if not allequal([im.origin for im in self.images]):
-                raise ValueError('Composite images with different origins not supported')
-            else:
-                origin = self.images[0].origin
 
-            ims = [(im.make_image(), im.ox, im.oy) for im in self.images]
-            im = _image.from_images(self.bbox.height(), self.bbox.width(), ims)
+            mag = renderer.get_image_magnification()
+            ims = [(im.make_image(mag), im.ox*mag, im.oy*mag)
+                   for im in self.images]
+            im = _image.from_images(self.bbox.height()*mag,
+                                    self.bbox.width()*mag,
+                                    ims)
             im.is_grayscale = False
             l, b, w, h = self.bbox.get_bounds()
-            renderer.draw_image(0, 0, im, origin, self.bbox)
-
+            renderer.draw_image(l, b, im, self.bbox)
 
 
         # render the axes
@@ -522,7 +612,7 @@ class Figure(Artist):
         self._cachedRenderer = renderer
 
         self.canvas.draw_event(renderer)
-        
+
     def draw_artist(self, a):
         'draw artist only -- this is available only after the figure is drawn'
         assert self._cachedRenderer is not None
@@ -562,7 +652,19 @@ class Figure(Artist):
         (0,0) is the left, bottom of the figure and 1,1 is the right,
         top.
 
-        The legend instance is returned
+        The legend instance is returned.  The following kwargs are supported:
+
+        isaxes=True           # whether this is an axes legend
+        numpoints = 4         # the number of points in the legend line
+        prop = FontProperties(size='smaller')  # the font property
+        pad = 0.2             # the fractional whitespace inside the legend border
+        markerscale = 0.6     # the relative size of legend markers vs. original
+        shadow                # if True, draw a shadow behind legend
+        labelsep = 0.005     # the vertical space between the legend entries
+        handlelen = 0.05     # the length of the legend lines
+        handletextsep = 0.02 # the space between the legend line and legend text
+        axespad = 0.02       # the border between the axes and legend edge
+
         """
 
 
@@ -576,6 +678,9 @@ class Figure(Artist):
         """
         Add text to figure at location x,y (relative 0-1 coords) See
         the help for Axis text for the meaning of the other arguments
+
+        kwargs control the Text properties:
+        %(Text)s
         """
 
         override = _process_text_args({}, *args, **kwargs)
@@ -587,6 +692,7 @@ class Figure(Artist):
         self._set_artist_props(t)
         self.texts.append(t)
         return t
+    text.__doc__ = dedent(text.__doc__) % artist.kwdocd
 
     def _set_artist_props(self, a):
         if a!= self:
@@ -596,10 +702,14 @@ class Figure(Artist):
     def gca(self, **kwargs):
         """
         Return the current axes, creating one if necessary
+
+        The following kwargs are supported
+        %(Axes)s
         """
         ax = self._axstack()
         if ax is not None: return ax
         return self.add_subplot(111, **kwargs)
+    gca.__doc__ = dedent(gca.__doc__) % artist.kwdocd
 
     def sca(self, a):
         'Set the current axes to be a and return a'
@@ -614,19 +724,32 @@ class Figure(Artist):
 
     def savefig(self, *args, **kwargs):
         """
-        SAVEFIG(fname, dpi=150, facecolor='w', edgecolor='w',
-        orientation='portrait'):
+        SAVEFIG(fname, dpi=None, facecolor='w', edgecolor='w',
+        orientation='portrait', papertype=None, format=None):
 
-        Save the current figure to filename fname.  dpi is the resolution
-        in dots per inch.
+        Save the current figure.
 
-        Output file types currently supported are jpeg and png and will be
-        deduced by the extension to fname
+        fname - the filename to save the current figure to.  The
+                output formats supported depend on the backend being
+                used.  and are deduced by the extension to fname.
+                Possibilities are eps, jpeg, pdf, png, ps, svg.  fname
+                can also be a file or file-like object - cairo backend
+                only.  dpi - is the resolution in dots per inch.  If
+                None it will default to the value savefig.dpi in the
+                matplotlibrc file
 
-        facecolor and edgecolor are the colors os the figure rectangle
+        facecolor and edgecolor are the colors of the figure rectangle
 
         orientation is either 'landscape' or 'portrait' - not supported on
-        all backends; currently only on postscript output.
+        all backends; currently only on postscript output
+
+        papertype is is one of 'letter', 'legal', 'executive', 'ledger', 'a0'
+        through 'a10', or 'b0' through 'b10' - only supported for postscript
+        output
+
+        format - one of 'pdf', 'png', 'ps', 'svg'. It is used to specify the
+                 output when fname is a file or file-like object - cairo
+                 backend only.
         """
 
         for key in ('dpi', 'facecolor', 'edgecolor'):
@@ -635,93 +758,26 @@ class Figure(Artist):
 
         self.canvas.print_figure(*args, **kwargs)
 
-
-    def colorbar(self, mappable, tickfmt='%1.1f', cax=None, orientation='vertical'):
-        """
-        Create a colorbar for mappable image
-
-        mappable is the cm.ScalarMappable instance that you want to
-        colorbar to apply to, eg an Image as returned by imshow or a
-        PatchCollection as returned by scatter or pcolor.
-        
-        tickfmt is a format string to format the colorbar ticks
-
-        cax is a colorbar axes instance in which the colorbar will be
-        placed.  If None, as default axesd will be created resizing the
-        current aqxes to make room for it.  If not None, the supplied axes
-        will be used and the other axes positions will be unchanged.
-
-        orientation is the colorbar orientation: one of 'vertical' | 'horizontal'
-        return value is the colorbar axes instance
-        """
-
-        if orientation not in ('horizontal', 'vertical'):
-            raise ValueError('Orientation must be horizontal or vertical')
-
-        if isinstance(mappable, FigureImage) and cax is None:
-            raise TypeError('Colorbars for figure images currently not supported unless you provide a colorbar axes in cax')
-
-
+    def colorbar(self, mappable, cax=None, **kw):
+        orientation = kw.get('orientation', 'vertical')
         ax = self.gca()
-
-        cmap = mappable.cmap
-        norm = mappable.norm
-
-        if norm.vmin is None or norm.vmax is None:
-            mappable.autoscale()
-        cmin = norm.vmin
-        cmax = norm.vmax
-
         if cax is None:
-            l,b,w,h = ax.get_position()
-            if orientation=='vertical':
-                neww = 0.8*w
-                ax.set_position((l,b,neww,h))
-                cax = self.add_axes([l + 0.9*w, b, 0.1*w, h])
-            else:
-                newh = 0.8*h
-                ax.set_position((l,b+0.2*h,w,newh))
-                cax = self.add_axes([l, b, w, 0.1*h])
-
-        else:
-            if not isinstance(cax, Axes):
-                raise TypeError('Expected an Axes instance for cax')
-
-        N = cmap.N
-
-        c = linspace(cmin, cmax, N)
-        C = array([c,c])
-
-        if orientation=='vertical':
-            C = transpose(C)
-
-        if orientation=='vertical':
-            extent=(0, 1, cmin, cmax)
-        else:
-            extent=(cmin, cmax, 0, 1)
-        coll = cax.imshow(C,
-                          interpolation='nearest',
-                          #interpolation='bilinear',
-                          origin='lower',
-                          cmap=cmap, norm=norm,
-                          extent=extent)
-        mappable.add_observer(coll)
-        mappable.set_colorbar(coll, cax)
-
-        if orientation=='vertical':
-            cax.set_xticks([])
-            cax.yaxis.tick_right()
-            cax.yaxis.set_major_formatter(FormatStrFormatter(tickfmt))
-        else:
-            cax.set_yticks([])
-            cax.xaxis.set_major_formatter(FormatStrFormatter(tickfmt))
-
+            cax, kw = cbar.make_axes(ax, **kw)
+        cb = cbar.Colorbar(cax, mappable, **kw)
+        mappable.add_observer(cb)
+        mappable.set_colorbar(cb, cax)
         self.sca(ax)
-        return cax
+        return cb
+    colorbar.__doc__ =  '''
+        Create a colorbar for a ScalarMappable instance.
 
+        Documentation for the pylab thin wrapper: %s
+        '''% cbar.colorbar_doc
 
     def subplots_adjust(self, *args, **kwargs):
         """
+        subplots_adjust(self, left=None, bottom=None, right=None, top=None,
+                        wspace=None, hspace=None)
         fig.subplots_adjust(left=None, bottom=None, right=None, wspace=None, hspace=None):
         Update the SubplotParams with kwargs (defaulting to rc where
         None) and update the subplot locations
@@ -729,26 +785,51 @@ class Figure(Artist):
         self.subplotpars.update(*args, **kwargs)
         import matplotlib.axes
         for ax in self.axes:
-            if not isinstance(ax, matplotlib.axes.Subplot): continue
-            ax.update_params()
-            ax.set_position([ax.figLeft, ax.figBottom, ax.figW, ax.figH])
+            if not isinstance(ax, matplotlib.axes.Subplot):
+                # Check if sharing a subplots axis
+                if ax._sharex is not None and isinstance(ax._sharex, matplotlib.axes.Subplot):
+                    ax._sharex.update_params()
+                    ax.set_position([ax._sharex.figLeft, ax._sharex.figBottom, ax._sharex.figW, ax._sharex.figH])
+                elif ax._sharey is not None and isinstance(ax._sharey, matplotlib.axes.Subplot):
+                    ax._sharey.update_params()
+                    ax.set_position([ax._sharey.figLeft, ax._sharey.figBottom, ax._sharey.figW, ax._sharey.figH])
+            else:
+                ax.update_params()
+                ax.set_position([ax.figLeft, ax.figBottom, ax.figW, ax.figH])
 
 
 
-def figaspect(arr):
+def figaspect(arg):
     """
-    Determine the width and height for a figure that would fit array
-    preserving aspcect ratio.  The figure width, height in inches are
-    returned.  Be sure to create an axes with equal with and height, eg
+    Create a figure with specified aspect ratio.  If arg is a number,
+    use that aspect ratio.  If arg is an array, figaspect will
+    determine the width and height for a figure that would fit array
+    preserving aspect ratio.  The figure width, height in inches are
+    returned.  Be sure to create an axes with equal with and height,
+    eg
 
-    w, h = figaspect(A)
-    fig = Figure(figsize=(w,h))
-    ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
-    ax.imshow(A, **kwargs)
+    Example usage:
+
+      # make a figure twice as tall as it is wide
+      w, h = figaspect(2.)
+      fig = Figure(figsize=(w,h))
+      ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
+      ax.imshow(A, **kwargs)
+
+
+      # make a figure with the proper aspect for an array
+      A = rand(5,3)
+      w, h = figaspect(A)
+      fig = Figure(figsize=(w,h))
+      ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
+      ax.imshow(A, **kwargs)
 
     Thanks to Fernando Perez for this function
     """
 
+    isarray = hasattr(arg, 'shape')
+
+
     # min/max sizes to respect when autoscaling.  If John likes the idea, they
     # could become rc parameters, for now they're hardwired.
     figsize_min = array((4.0,2.0)) # min length for width/height
@@ -757,8 +838,11 @@ def figaspect(arr):
     #figsize_max = rcParams['figure.figsize_max']
 
     # Extract the aspect ratio of the array
-    nr,nc = arr.shape[:2]
-    arr_ratio = float(nr)/nc
+    if isarray:
+        nr,nc = arg.shape[:2]
+        arr_ratio = float(nr)/nc
+    else:
+        arr_ratio = float(arg)
 
     # Height of user figure defaults
     fig_height = rcParams['figure.figsize'][1]
@@ -777,3 +861,4 @@ def figaspect(arr):
     newsize = clip(newsize,figsize_min,figsize_max)
     return newsize
 
+artist.kwdocd['Figure'] = artist.kwdoc(Figure)
