@@ -11,29 +11,18 @@ import sys, math, warnings
 import agg
 from numerix import Float, alltrue, arange, array, logical_and,\
      nonzero, searchsorted, take, asarray, ones, where, less, ravel, \
-     greater, logical_and, cos, sin, pi,\
-     compress, zeros, concatenate, cumsum
+     greater, logical_and, cos, sin, pi, sqrt, less_equal, \
+     compress, zeros, concatenate, cumsum, typecode, NewAxis
 import numerix.ma as ma
 from matplotlib import verbose
+import artist
 from artist import Artist, setp
-from cbook import iterable, is_string_like
+from cbook import iterable, is_string_like, is_numlike
 from colors import colorConverter
-from patches import bbox_artist
 
 from transforms import lbwh_to_bbox, LOG10
 from matplotlib import rcParams
-
 TICKLEFT, TICKRIGHT, TICKUP, TICKDOWN = range(4)
-lineStyles  = {'-':1, '--':1, '-.':1, ':':1, 'steps':1, 'None':1}
-lineMarkers =    {'.':1, ',':1, 'o':1, '^':1, 'v':1, '<':1, '>':1, 's':1,
-                  '+':1, 'x':1, 'd':1, 'D':1, '|':1, '_':1, 'h':1, 'H':1,
-                  'p':1, '1':1, '2':1, '3':1, '4':1,
-                  TICKLEFT:1,
-                  TICKRIGHT:1,
-                  TICKUP:1,
-                  TICKDOWN:1,
-                  'None':1
-                  }
 
 def unmasked_index_ranges(mask, compressed = True):
     '''
@@ -45,20 +34,21 @@ def unmasked_index_ranges(mask, compressed = True):
     If optional argument compressed is False, it returns the
     start and stop indices into the original array, not the
     compressed array.
-    In either case, an empty array is returned if there are no
-    unmasked values.
+    Returns None if there are no unmasked values.
 
     Example:
 
     y = ma.array(arange(5), mask = [0,0,1,0,0])
-    ii = unmasked_index_ranges(y.mask())
+    #ii = unmasked_index_ranges(y.mask())
+    ii = unmasked_index_ranges(ma.getmask(y))
         # returns [[0,2,] [2,4,]]
 
     y.compressed().filled()[ii[1,0]:ii[1,1]]
         # returns array [3,4,]
         # (The 'filled()' method converts the masked array to a numerix array.)
 
-    i0, i1 = unmasked_index_ranges(y.mask(), compressed=False)
+    #i0, i1 = unmasked_index_ranges(y.mask(), compressed=False)
+    i0, i1 = unmasked_index_ranges(ma.getmask(y), compressed=False)
         # returns [[0,3,] [2,5,]]
 
     y.filled()[ii[1,0]:ii[1,1]]
@@ -71,20 +61,15 @@ def unmasked_index_ranges(mask, compressed = True):
     i0 = compress(mdif == -1, indices)
     i1 = compress(mdif == 1, indices)
     assert len(i0) == len(i1)
+    if len(i1) == 0:
+        return None
     if not compressed:
-        if len(i1):
-            return concatenate((i0[:, ma.NewAxis], i1[:, ma.NewAxis]), axis=1)
-        else:
-            return zeros((0,0), 'i')
+        return concatenate((i0[:, NewAxis], i1[:, NewAxis]), axis=1)
     seglengths = i1 - i0
     breakpoints = cumsum(seglengths)
-    try:
-        ic0 = concatenate(((0,), breakpoints[:-1]))
-        ic1 = breakpoints
-        return concatenate((ic0[:, ma.NewAxis], ic1[:, ma.NewAxis]), axis=1)
-    except:
-        return zeros((0,0), 'i')
-
+    ic0 = concatenate(((0,), breakpoints[:-1]))
+    ic1 = breakpoints
+    return concatenate((ic0[:, NewAxis], ic1[:, NewAxis]), axis=1)
 
 
 class Line2D(Artist):
@@ -94,7 +79,10 @@ class Line2D(Artist):
         '-.'   : '_draw_dash_dot',
         ':'    : '_draw_dotted',
         'steps': '_draw_steps',
-        'None' : '_draw_nothing'}
+        'None' : '_draw_nothing',
+        ' '    : '_draw_nothing',
+        ''     : '_draw_nothing',
+    }
 
     _markers =  {
         '.'  : '_draw_point',
@@ -122,8 +110,12 @@ class Line2D(Artist):
         TICKRIGHT   : '_draw_tickright',
         TICKUP      : '_draw_tickup',
         TICKDOWN    : '_draw_tickdown',
-        'None' : '_draw_nothing'
-        }
+        'None' : '_draw_nothing',
+        ' ' : '_draw_nothing',
+        '' : '_draw_nothing',
+    }
+
+    filled_markers = ('o', '^', 'v', '<', '>', 's', 'd', 'D', 'h', 'H', 'p')
 
     zorder = 2
     validCap = ('butt', 'round', 'projecting')
@@ -146,18 +138,37 @@ class Line2D(Artist):
                  **kwargs
                  ):
         """
-        xdata is a sequence of x data
-        ydata is a sequence of y data
-        linewidth is the width of the line in points
-        linestyle is one of lineStyles
-        marker is one of lineMarkers or None
-        markersize is the size of the marker in points
-        markeredgecolor  and markerfacecolor can be any color arg
-        markeredgewidth is the size of the markter edge in points
-        dash_capstyle set the capstyle for dashed line
-        solid_capstyle set the capstyle for solid line
-        dash_joinstyle set the joinstyle for dashed line
-        solid_joinstyle set the joinstyle for solid line
+        Create a Line2D instance with x and y data in sequences xdata,
+        ydata
+
+        The kwargs are Line2D properties:
+          alpha: float
+          animated: [True | False]
+          antialiased or aa: [True | False]
+          clip_box: a matplotlib.transform.Bbox instance
+          clip_on: [True | False]
+          color or c: any matplotlib color
+          dash_capstyle: ['butt' | 'round' | 'projecting']
+          dash_joinstyle: ['miter' | 'round' | 'bevel']
+          dashes: sequence of on/off ink in points
+          data: (array xdata, array ydata)
+          figure: a matplotlib.figure.Figure instance
+          label: any string
+          linestyle or ls: [ '-' | '--' | '-.' | ':' | 'steps' | 'None' | ' ' | '' ]
+          linewidth or lw: float value in points
+          lod: [True | False]
+          marker: [ '+' | ',' | '.' | '1' | '2' | '3' | '4'
+          markeredgecolor or mec: any matplotlib color
+          markeredgewidth or mew: float value in points
+          markerfacecolor or mfc: any matplotlib color
+          markersize or ms: float
+          solid_capstyle: ['butt' | 'round' |  'projecting']
+          solid_joinstyle: ['miter' | 'round' | 'bevel']
+          transform: a matplotlib.transform transformation instance
+          visible: [True | False]
+          xdata: array
+          ydata: array
+          zorder: any number
         """
         Artist.__init__(self)
 
@@ -173,9 +184,9 @@ class Line2D(Artist):
         if marker is None      : marker=rcParams['lines.marker']
         if color is None       : color=rcParams['lines.color']
         if markeredgecolor is None :
-            markeredgecolor=rcParams['lines.markeredgecolor']
+            markeredgecolor='auto'
         if markerfacecolor is None :
-            markerfacecolor=rcParams['lines.markerfacecolor']
+            markerfacecolor='auto'
         if markeredgewidth is None :
             markeredgewidth=rcParams['lines.markeredgewidth']
 
@@ -192,32 +203,66 @@ class Line2D(Artist):
         self.set_solid_joinstyle(solid_joinstyle)
 
 
-        self._linestyle = linestyle
-        self._linewidth = linewidth
-        self._color = color
-        self._antialiased = antialiased
-        self._markersize = markersize
+        self.set_linestyle(linestyle)
+        self.set_linewidth(linewidth)
+        self.set_color(color)
+        self.set_marker(marker)
+        self.set_antialiased(antialiased)
+        self.set_markersize(markersize)
         self._dashSeq = None
 
 
-        self._markerfacecolor = markerfacecolor
-        self._markeredgecolor = markeredgecolor
-        self._markeredgewidth = markeredgewidth
+        self.set_markerfacecolor(markerfacecolor)
+        self.set_markeredgecolor(markeredgecolor)
+        self.set_markeredgewidth(markeredgewidth)
+        self._point_size_reduction = 0.5
 
         self.verticalOffset = None
-        self._useDataClipping = rcParams['lines.data_clipping']
+
+        # update kwargs before updating data to give the caller a
+        # chance to init axes (and hence unit support)
+        self.update(kwargs)
+
         self.set_data(xdata, ydata)
-
-        if not self._lineStyles.has_key(linestyle):
-            raise ValueError('Unrecognized line style %s, %s' %( linestyle, type(linestyle)))
-        if not self._markers.has_key(marker):
-            raise ValueError('Unrecognized marker style %s, %s'%( marker, type(marker)))
-
-        self.set_marker(marker)
-
         self._logcache = None
 
-        if len(kwargs): setp(self, **kwargs)
+    def pick(self, mouseevent):
+        """
+        If mouseevent is over data that satisifies the picker, fire
+        off a backend_bases.PickEvent with the additional attribute "ind"
+        which is a sequence of indices into the data that meet the criteria
+        """
+        if not self.pickable(): return
+
+        if self._newstyle:
+            # transform in backend
+            x = self._x
+            y = self._y
+        else:
+            x, y = self._get_plottable()
+
+        picker = self.get_picker()
+        if is_numlike(picker):
+            eps = picker # epsilon tolerance in points
+            if len(x)==0: return
+            xt, yt = self.get_transform().numerix_x_y(x, y)
+
+            d = sqrt((xt-mouseevent.x)**2. + (yt-mouseevent.y)**2.)
+            pixels = self.figure.dpi.get()/72. * eps
+            ind = nonzero(less_equal(d, pixels))
+            if 0:
+                print 'xt', xt, mouseevent.x
+                print 'yt', yt, mouseevent.y
+                print 'd', (xt-mouseevent.x)**2., (yt-mouseevent.y)**2.
+                print d, pixels, ind
+            if len(ind):
+                self.figure.canvas.pick_event(mouseevent, self, ind=ind)
+        elif callable(picker):
+            hit, props = picker(self, mouseevent)
+            if hit:
+                self.figure.canvas.pick_event(mouseevent, self, **props)
+
+
 
     def get_window_extent(self, renderer):
         self._newstyle = hasattr(renderer, 'draw_markers')
@@ -225,11 +270,11 @@ class Line2D(Artist):
             x = self._x
             y = self._y
         else:
-            x, y = self._get_numeric_clipped_data_in_range()
+            x, y = self._get_plottable()
 
 
-        x, y = self._transform.numerix_x_y(x, y)
-        #x, y = self._transform.seq_x_y(x, y)
+        x, y = self.get_transform().numerix_x_y(x, y)
+        #x, y = self.get_transform().seq_x_y(x, y)
 
         left = min(x)
         bottom = min(y)
@@ -246,6 +291,13 @@ class Line2D(Artist):
         return lbwh_to_bbox( left, bottom, width, height)
 
 
+    def set_axes(self, ax):
+        Artist.set_axes(self, ax)
+        if ax.xaxis is not None:
+            self._xcid = ax.xaxis.callbacks.connect('units', self.recache)
+        if ax.yaxis is not None:
+            self._ycid = ax.yaxis.callbacks.connect('units', self.recache)
+
     def set_data(self, *args):
         """
         Set the x and y data
@@ -258,12 +310,15 @@ class Line2D(Artist):
         else:
             x, y = args
 
+        self._xorig = x
+        self._yorig = y
+        self.recache()
 
-        try: del self._xc, self._yc
-        except AttributeError: pass
-
-        self._x_orig = x
-        self._y_orig = y
+    def recache(self):
+        #if self.axes is None: print 'recache no axes'
+        #else: print 'recache units', self.axes.xaxis.units, self.axes.yaxis.units
+        x = ma.asarray(self.convert_xunits(self._xorig), Float)
+        y = ma.asarray(self.convert_yunits(self._yorig), Float)
 
         x = ma.ravel(x)
         y = ma.ravel(y)
@@ -278,7 +333,7 @@ class Line2D(Artist):
         mx = ma.getmask(x)
         my = ma.getmask(y)
         mask = ma.mask_or(mx, my)
-        if mask is not None:
+        if mask is not ma.nomask:
             x = ma.masked_array(x, mask=mask).compressed()
             y = ma.masked_array(y, mask=mask).compressed()
             self._segments = unmasked_index_ranges(mask)
@@ -288,17 +343,8 @@ class Line2D(Artist):
         self._x = asarray(x, Float)
         self._y = asarray(y, Float)
 
-        if self._useDataClipping: self._xsorted = self._is_sorted(self._x)
-
         self._logcache = None
 
-    def set_data_clipping(self, b):
-        """
-        b is a boolean that sets whether data clipping is on
-
-        ACCEPTS: [True | False]
-        """
-        self._useDataClipping = b
 
 
     def _is_sorted(self, x):
@@ -306,19 +352,15 @@ class Line2D(Artist):
         if len(x)<2: return 1
         return alltrue(x[1:]-x[0:-1]>=0)
 
-    def _get_numeric_clipped_data_in_range(self):
-        # if the x or y clip is set, only plot the points in the
-        # clipping region.  If log scale is set, only pos data will be
-        # returned
+    def _get_plottable(self):
+        # If log scale is set, only pos data will be returned
 
-        try: self._xc, self._yc
-        except AttributeError: x, y = self._x, self._y
-        else: x, y = self._xc, self._yc
+        x, y = self._x, self._y
 
-        try: logx = self._transform.get_funcx().get_type()==LOG10
+        try: logx = self.get_transform().get_funcx().get_type()==LOG10
         except RuntimeError: logx = False  # non-separable
 
-        try: logy = self._transform.get_funcy().get_type()==LOG10
+        try: logy = self.get_transform().get_funcy().get_type()==LOG10
         except RuntimeError: logy = False  # non-separable
 
         if not logx and not logy:
@@ -345,19 +387,19 @@ class Line2D(Artist):
         self._logcache = logx, logy, x, y
         return x, y
 
+
     def draw(self, renderer):
         #renderer.open_group('line2d')
 
         if not self._visible: return
         self._newstyle = hasattr(renderer, 'draw_markers')
         gc = renderer.new_gc()
+        self._set_gc_clip(gc)
+
         gc.set_foreground(self._color)
         gc.set_antialiased(self._antialiased)
         gc.set_linewidth(self._linewidth)
         gc.set_alpha(self._alpha)
-        if self.get_clip_on():
-            gc.set_clip_rectangle(self.clipbox.get_bounds())
-
         if self.is_dashed():
             cap = self._dashcapstyle
             join = self._dashjoinstyle
@@ -372,9 +414,9 @@ class Line2D(Artist):
             xt = self._x
             yt = self._y
         else:
-            x, y = self._get_numeric_clipped_data_in_range()
+            x, y = self._get_plottable()
             if len(x)==0: return
-            xt, yt = self._transform.numerix_x_y(x, y)
+            xt, yt = self.get_transform().numerix_x_y(x, y)
 
 
 
@@ -384,21 +426,29 @@ class Line2D(Artist):
         if self._segments is not None:
             for ii in self._segments:
                 lineFunc(renderer, gc, xt[ii[0]:ii[1]], yt[ii[0]:ii[1]])
+
         else:
             lineFunc(renderer, gc, xt, yt)
 
 
         if self._marker is not None:
+
             gc = renderer.new_gc()
-            gc.set_foreground(self._markeredgecolor)
+            self._set_gc_clip(gc)
+            if (is_string_like(self._markeredgecolor) and
+                self._markeredgecolor == 'auto'):
+                if self._marker in self.filled_markers:
+                    gc.set_foreground('k')
+                else:
+                    gc.set_foreground(self._color)
+            else:
+                gc.set_foreground(self._markeredgecolor)
             gc.set_linewidth(self._markeredgewidth)
-            if self.get_clip_on():
-                gc.set_clip_rectangle(self.clipbox.get_bounds())
+            gc.set_alpha(self._alpha)
             funcname = self._markers.get(self._marker, '_draw_nothing')
             markerFunc = getattr(self, funcname)
             markerFunc(renderer, gc, xt, yt)
 
-        #if 1: bbox_artist(self, renderer)
         #renderer.close_group('line2d')
 
     def get_antialiased(self): return self._antialiased
@@ -411,69 +461,24 @@ class Line2D(Artist):
     def get_markeredgewidth(self): return self._markeredgewidth
     def get_markerfacecolor(self): return self._markerfacecolor
     def get_markersize(self): return self._markersize
-    def get_xdata(self, valid_only = False):
-        if valid_only:
-            return self._x
-        return self._x_orig
-    def get_ydata(self, valid_only = False):
-        if valid_only:
-            return self._y
-        return self._y_orig
 
+    def get_xdata(self, orig=True):
+        """
+        return the xdata; if orig is true return the original data,
+        else the processed data
+        """
+        if orig:
+            return self._xorig
+        return self._x
 
-    def _set_clip(self):
-
-
-        if not self._useDataClipping: return
-        #self._logcache = None
-
-        try: self._xmin, self._xmax
-        except AttributeError: indx = arange(len(self._x))
-        else:
-            if not hasattr(self, '_xsorted'):
-                self._xsorted = self._is_sorted(self._x)
-            if len(self._x)==1:
-                indx = [0]
-            elif self._xsorted:
-                # for really long signals, if we know they are sorted
-                # on x we can save a lot of time using search sorted
-                # since the alternative approach requires 3 O(len(x) ) ops
-                indMin, indMax = searchsorted(
-                    self._x, array([self._xmin, self._xmax]))
-                indMin = max(0, indMin-1)
-                indMax = min(indMax+1, len(self._x))
-                skip = 0
-                if self._lod:
-                    # if level of detail is on, decimate the data
-                    # based on pixel width
-                    raise NotImplementedError('LOD deprecated')
-                    l, b, w, h = self.get_window_extent().get_bounds()
-                    skip = int((indMax-indMin)/w)
-                if skip>0:  indx = arange(indMin, indMax, skip)
-                else: indx = arange(indMin, indMax)
-            else:
-                indx = nonzero(
-                    logical_and( self._x>=self._xmin,
-                                 self._x<=self._xmax ))
-
-        self._xc = take(self._x, indx)
-        self._yc = take(self._y, indx)
-
-        # y data clipping for connected lines can introduce horizontal
-        # line artifacts near the clip region.  If you really need y
-        # clipping for efficiency, consider using plot(y,x) instead.
-        if ( self._yc.shape==self._xc.shape and
-             self._linestyle is None):
-            try: self._ymin, self._ymax
-            except AttributeError: indy = arange(len(self._yc))
-            else: indy = nonzero(
-                logical_and(self._yc>=self._ymin,
-                                  self._yc<=self._ymax ))
-        else:
-            indy = arange(len(self._yc))
-
-        self._xc = take(self._xc, indy)
-        self._yc = take(self._yc, indy)
+    def get_ydata(self, orig=True):
+        """
+        return the ydata; if orig is true return the original data,
+        else the processed data
+        """
+        if orig:
+            return self._yorig
+        return self._y
 
     def set_antialiased(self, b):
         """
@@ -487,7 +492,7 @@ class Line2D(Artist):
         """
         Set the color of the line
 
-        ACCEPTS: any matplotlib color - see help(colors)
+        ACCEPTS: any matplotlib color
         """
         self._color = color
 
@@ -499,29 +504,44 @@ class Line2D(Artist):
         """
         self._linewidth = w
 
-    def set_linestyle(self, s):
+    def set_linestyle(self, linestyle):
         """
         Set the linestyle of the line
 
-        ACCEPTS: [ '-' | '--' | '-.' | ':' | 'steps' | 'None' ]
+        ACCEPTS: [ '-' | '--' | '-.' | ':' | 'steps' | 'None' | ' ' | '' ]
         """
-        self._linestyle = s
-
+        if linestyle not in self._lineStyles:
+            verbose.report('Unrecognized line style %s, %s' %
+                                            (linestyle, type(linestyle)))
+        if linestyle in [' ','']:
+            linestyle = 'None'
+        self._linestyle = linestyle
+        self._lineFunc = self._lineStyles[linestyle]
 
     def set_marker(self, marker):
         """
         Set the line marker
 
-        ACCEPTS: [ '+' | ',' | '.' | '1' | '2' | '3' | '4' | '<' | '>' | 'D' | 'H' | '^' | '_' | 'd' | 'h' | 'o' | 'p' | 's' | 'v' | 'x' | '|' ]
+        ACCEPTS: [ '+' | ',' | '.' | '1' | '2' | '3' | '4'
+                 | '<' | '>' | 'D' | 'H' | '^' | '_' | 'd'
+                 | 'h' | 'o' | 'p' | 's' | 'v' | 'x' | '|'
+                 | TICKUP | TICKDOWN | TICKLEFT | TICKRIGHT
+                 | 'None' | ' ' | '' ]
 
         """
+        if marker not in self._markers:
+            verbose.report('Unrecognized marker style %s, %s' %
+                                            (marker, type(marker)))
+        if marker in [' ','']:
+            marker = 'None'
         self._marker = marker
+        self._markerFunc = self._markers[marker]
 
     def set_markeredgecolor(self, ec):
         """
         Set the marker edge color
 
-        ACCEPTS: any matplotlib color - see help(colors)
+        ACCEPTS: any matplotlib color
         """
         self._markeredgecolor = ec
 
@@ -537,7 +557,7 @@ class Line2D(Artist):
         """
         Set the marker face color
 
-        ACCEPTS: any matplotlib color - see help(colors)
+        ACCEPTS: any matplotlib color
         """
         self._markerfacecolor = fc
 
@@ -569,45 +589,16 @@ class Line2D(Artist):
 
         self.set_data(self.get_xdata(), y)
 
-    def set_xclip(self, *args):
-        """
-        Set the x clipping range for data clipping to xmin, xmax
-
-        ACCEPTS: (xmin, xmax)
-        """
-        if len(args)==1:
-            xmin, xmax = args[0]
-        else:
-            xmin, xmax = args
-
-        if xmax<xmin: xmax, xmin = xmin, xmax
-        self._xmin, self._xmax = xmin, xmax
-        self._set_clip()
-
-
-
-    def set_yclip(self, *args):
-        """
-        Set the y clipping range for data clipping to ymin, ymax
-
-        ACCEPTS: (ymin, ymax)
-        """
-        if len(args)==1:
-            ymin, ymax = args[0]
-        else:
-            ymin, ymax = args
-
-        if ymax<ymin: ymax, ymin = ymin, ymax
-        self._ymin, self._ymax = ymin, ymax
-        self._set_clip()
 
     def set_dashes(self, seq):
         """
-        Set the dash sequence, sequence of dashes with on off ink in points
+        Set the dash sequence, sequence of dashes with on off ink in
+        points.  If seq is empty or if seq = (None, None), the
+        linestyle will be set to solid.
 
         ACCEPTS: sequence of on/off ink in points
         """
-        if seq == (None, None):
+        if seq == (None, None) or len(seq)==0:
             self.set_linestyle('-')
         else:
             self.set_linestyle('--')
@@ -619,14 +610,14 @@ class Line2D(Artist):
     def _draw_steps(self, renderer, gc, xt, yt):
         siz=len(xt)
         if siz<2: return
-        xt2=ones((2*siz,), xt.typecode())
+        xt2=ones((2*siz,), typecode(xt))
         xt2[0:-1:2], xt2[1:-1:2], xt2[-1]=xt, xt[1:], xt[-1]
-        yt2=ones((2*siz,), yt.typecode())
+        yt2=ones((2*siz,), typecode(yt))
         yt2[0:-1:2], yt2[1::2]=yt, yt
         gc.set_linestyle('solid')
 
         if self._newstyle:
-            renderer.draw_lines(gc, xt2, yt2, self._transform)
+            renderer.draw_lines(gc, xt2, yt2, self.get_transform())
         else:
             renderer.draw_lines(gc, xt2, yt2)
 
@@ -634,7 +625,7 @@ class Line2D(Artist):
         if len(xt)<2: return
         gc.set_linestyle('solid')
         if self._newstyle:
-            renderer.draw_lines(gc, xt, yt, self._transform)
+            renderer.draw_lines(gc, xt, yt, self.get_transform())
         else:
             renderer.draw_lines(gc, xt, yt)
 
@@ -646,7 +637,7 @@ class Line2D(Artist):
             gc.set_dashes(0, self._dashSeq)
 
         if self._newstyle:
-            renderer.draw_lines(gc, xt, yt, self._transform)
+            renderer.draw_lines(gc, xt, yt, self.get_transform())
         else:
             renderer.draw_lines(gc, xt, yt)
 
@@ -655,7 +646,7 @@ class Line2D(Artist):
         if len(xt)<2: return
         gc.set_linestyle('dashdot')
         if self._newstyle:
-            renderer.draw_lines(gc, xt, yt, self._transform)
+            renderer.draw_lines(gc, xt, yt, self.get_transform())
         else:
             renderer.draw_lines(gc, xt, yt)
 
@@ -664,22 +655,21 @@ class Line2D(Artist):
         if len(xt)<2: return
         gc.set_linestyle('dotted')
         if self._newstyle:
-            renderer.draw_lines(gc, xt, yt, self._transform)
+            renderer.draw_lines(gc, xt, yt, self.get_transform())
         else:
             renderer.draw_lines(gc, xt, yt)
 
     def _draw_point(self, renderer, gc, xt, yt):
-        if self._newstyle:
-            rgbFace = self._get_rgb_face()
-            path = agg.path_storage()
-            path.move_to(-0.5, -0.5)
-            path.line_to(-0.5, 0.5)
-            path.line_to(0.5, 0.5)
-            path.line_to(0.5, -0.5)
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+
+        r = 0.5 * renderer.points_to_pixels(self._markersize)
+        r *= self._point_size_reduction
+        gc.set_linewidth(0)
+        if r <= 0.5:
+            self._draw_pixel(renderer, gc, xt, yt)
+        elif r <= 2:
+            self._draw_hexagon1(renderer, gc, xt, yt, point=True)
         else:
-            for (x,y) in zip(xt, yt):
-                renderer.draw_arc(gc, None, x, y, 1, 1, 0.0, 360.0)
+            self._draw_circle(renderer, gc, xt, yt, point=True)
 
     def _draw_pixel(self, renderer, gc, xt, yt):
         if self._newstyle:
@@ -689,15 +679,18 @@ class Line2D(Artist):
             path.line_to(-0.5, 0.5)
             path.line_to(0.5, 0.5)
             path.line_to(0.5, -0.5)
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_point(gc, x, y)
 
 
-    def _draw_circle(self, renderer, gc, xt, yt):
+    def _draw_circle(self, renderer, gc, xt, yt, point=False):
+
+        w = renderer.points_to_pixels(self._markersize)
+        if point:
+            w *= self._point_size_reduction
 
-        w = h = renderer.points_to_pixels(self._markersize)
 
         rgbFace = self._get_rgb_face()
 
@@ -714,11 +707,11 @@ class Line2D(Artist):
                 path.line_to(x, y)
 
             path.end_poly()
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
-            for (x,y) in zip(xt, yt):
+            for (x,y) in zip(xt,yt):
                 renderer.draw_arc(gc, rgbFace,
-                                  x, y, w, h, 0.0, 360.0)
+                                  x, y, w, w, 0.0, 360.0, 0.0)
 
 
 
@@ -734,7 +727,7 @@ class Line2D(Artist):
             path.line_to(-offset, -offset)
             path.line_to(offset, -offset)
             path.end_poly()
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x, y+offset),
@@ -755,7 +748,7 @@ class Line2D(Artist):
             path.line_to(0, -offset)
             path.end_poly()
 
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x-offset, y+offset),
@@ -775,7 +768,7 @@ class Line2D(Artist):
             path.line_to(offset, offset)
             path.end_poly()
 
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x-offset, y),
@@ -793,7 +786,7 @@ class Line2D(Artist):
             path.line_to(-offset, -offset)
             path.line_to(-offset, offset)
             path.end_poly()
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x+offset, y),
@@ -817,7 +810,7 @@ class Line2D(Artist):
             path.line_to(offset, -offset)
             path.end_poly()
 
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
 
             for (x,y) in zip(xt, yt):
@@ -836,7 +829,7 @@ class Line2D(Artist):
             path.line_to(0, offset)
             path.end_poly()
 
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
 
 
@@ -859,7 +852,7 @@ class Line2D(Artist):
             path.line_to(-xoffset, 0)
             path.line_to(0, offset)
             path.end_poly()
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x+xoffset, y),
@@ -885,7 +878,7 @@ class Line2D(Artist):
             path.line_to(+offsetX1, offsetY1)
             path.end_poly()
 
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x, y+offset),
@@ -895,8 +888,10 @@ class Line2D(Artist):
                           (x+offsetX1, y+offsetY1))
                 renderer.draw_polygon(gc, rgbFace, verts)
 
-    def _draw_hexagon1(self, renderer, gc, xt, yt):
+    def _draw_hexagon1(self, renderer, gc, xt, yt, point=False):
         offset = 0.6*renderer.points_to_pixels(self._markersize)
+        if point:
+            offset *= self._point_size_reduction
         offsetX1 = offset*0.87
         offsetY1 = offset*0.5
         rgbFace = self._get_rgb_face()
@@ -910,7 +905,7 @@ class Line2D(Artist):
             path.line_to(offsetX1, -offsetY1)
             path.line_to(offsetX1, offsetY1)
             path.end_poly()
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x, y+offset),
@@ -936,7 +931,7 @@ class Line2D(Artist):
             path.line_to(offsetX1, -offsetY1)
             path.end_poly()
 
-            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x+offset, y),
@@ -953,7 +948,7 @@ class Line2D(Artist):
             path = agg.path_storage()
             path.move_to(0, -offset)
             path.line_to(0, offset)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y-offset, x, y+offset)
@@ -964,7 +959,7 @@ class Line2D(Artist):
             path = agg.path_storage()
             path.move_to(-offset, 0)
             path.line_to(offset, 0)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x-offset, y, x+offset, y)
@@ -975,7 +970,7 @@ class Line2D(Artist):
             path = agg.path_storage()
             path.move_to(-offset, 0.5)
             path.line_to(0, 0.5)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x-offset, y, x, y)
@@ -987,7 +982,7 @@ class Line2D(Artist):
             path = agg.path_storage()
             path.move_to(0, 0.5)
             path.line_to(offset, 0.5)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x+offset, y)
@@ -998,7 +993,7 @@ class Line2D(Artist):
             path = agg.path_storage()
             path.move_to(-0.5, 0)
             path.line_to(-0.5, offset)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x, y+offset)
@@ -1009,7 +1004,7 @@ class Line2D(Artist):
             path = agg.path_storage()
             path.move_to(-0.5, -offset)
             path.line_to(-0.5, 0)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y-offset, x, y)
@@ -1023,7 +1018,7 @@ class Line2D(Artist):
             path.line_to( offset, 0)
             path.move_to( 0, -offset)
             path.line_to( 0, offset)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x-offset, y, x+offset, y)
@@ -1041,7 +1036,7 @@ class Line2D(Artist):
             path.line_to(offset1, offset2)
             path.move_to(0, 0)
             path.line_to(-offset1, offset2)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x, y-offset)
@@ -1060,7 +1055,7 @@ class Line2D(Artist):
             path.line_to(offset1, -offset2)
             path.move_to(0, 0)
             path.line_to(-offset1, -offset2)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x, y+offset)
@@ -1079,7 +1074,7 @@ class Line2D(Artist):
             path.line_to(offset2, offset1)
             path.move_to(0, 0)
             path.line_to(offset2, -offset1)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x-offset, y)
@@ -1098,7 +1093,7 @@ class Line2D(Artist):
             path.line_to(-offset2, offset1)
             path.move_to(0, 0)
             path.line_to(-offset2, -offset1)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x+offset, y)
@@ -1114,7 +1109,7 @@ class Line2D(Artist):
             path.line_to(offset, offset)
             path.move_to(-offset, offset)
             path.line_to(offset, -offset)
-            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x-offset, y-offset, x+offset, y+offset)
@@ -1131,16 +1126,25 @@ class Line2D(Artist):
         self._markeredgecolor = other._markeredgecolor
         self._markeredgewidth = other._markeredgewidth
         self._dashSeq = other._dashSeq
+        self._dashcapstyle = other._dashcapstyle
+        self._dashjoinstyle = other._dashjoinstyle
+        self._solidcapstyle = other._solidcapstyle
+        self._solidjoinstyle = other._solidjoinstyle
 
         self._linestyle = other._linestyle
         self._marker = other._marker
-        self._useDataClipping = other._useDataClipping
+
 
     def _get_rgb_face(self):
         if (self._markerfacecolor is None or
             (is_string_like(self._markerfacecolor) and
-             self._markerfacecolor.lower()=='none') ): rgbFace = None
-        else: rgbFace = colorConverter.to_rgb(self._markerfacecolor)
+             self._markerfacecolor.lower()=='none') ):
+            rgbFace = None
+        elif (is_string_like(self._markerfacecolor) and
+              self._markerfacecolor.lower() == 'auto'):
+            rgbFace = colorConverter.to_rgb(self._color)
+        else:
+            rgbFace = colorConverter.to_rgb(self._markerfacecolor)
         return rgbFace
 
     # some aliases....
@@ -1227,7 +1231,8 @@ class Line2D(Artist):
         """
         s = s.lower()
         if s not in self.validJoin:
-            raise ValueError('set_dash_joinstyle passed "%s"; valid joinstyles are %s'%(s, self.validJoin))
+            raise ValueError('set_dash_joinstyle passed "%s";\n' % (s,)
+                  + 'valid joinstyles are %s' % (self.validJoin,))
         self._dashjoinstyle = s
 
     def set_solid_joinstyle(self, s):
@@ -1237,7 +1242,8 @@ class Line2D(Artist):
         """
         s = s.lower()
         if s not in self.validJoin:
-            raise ValueError('set_solid_joinstyle passed "%s"; valid joinstyles are %s'%(s, self.validJoin))
+            raise ValueError('set_solid_joinstyle passed "%s";\n' % (s,)
+                  + 'valid joinstyles are %s' % (self.validJoin,))
         self._solidjoinstyle = s
 
 
@@ -1260,7 +1266,8 @@ class Line2D(Artist):
         """
         s = s.lower()
         if s not in self.validCap:
-            raise ValueError('set_dash_capstyle passed "%s"; valid capstyles are %s'%(s, self.validJoin))
+            raise ValueError('set_dash_capstyle passed "%s";\n' % (s,)
+                  + 'valid capstyles are %s' % (self.validCap,))
 
         self._dashcapstyle = s
 
@@ -1272,7 +1279,8 @@ class Line2D(Artist):
         """
         s = s.lower()
         if s not in self.validCap:
-            raise ValueError('set_solid_capstyle passed "%s"; valid capstyles are %s'%(s, self.validJoin))
+            raise ValueError('set_solid_capstyle passed "%s";\n' % (s,)
+                  + 'valid capstyles are %s' % (self.validCap,))
 
         self._solidcapstyle = s
 
@@ -1293,3 +1301,8 @@ class Line2D(Artist):
         'return True if line is dashstyle'
         return self._linestyle in ('--', '-.', ':')
 
+
+lineStyles = Line2D._lineStyles
+lineMarkers = Line2D._markers
+
+artist.kwdocd['Line2D'] = artist.kwdoc(Line2D)
