@@ -13,17 +13,52 @@ from lines import Line2D
 from numerix import array
 from transforms import blend_xy_sep_transform
 
-import thread
+class LockDraw:
+    """
+    some widgets, like the cursor, draw onto the canvas, and this is not
+    desirable under all circumstaces, like when the toolbar is in
+    zoom-to-rect mode and drawing a rectangle.  The module level "lock"
+    allows someone to grab the lock and prevent other widgets from
+    drawing.  Use matplotlib.widgets.lock(someobj) to pr
+    """
+    def __init__(self):
+        self._owner = None
+
+    def __call__(self, o):
+        'reserve the lock for o'
+        if not self.available(o):
+            raise ValueError('already locked')
+        self._owner = o
+
+    def release(self, o):
+        'release the lock'
+        if not self.available(o):
+            raise ValueError('you do not own this lock')
+        self._owner = None
+
+    def available(self, o):
+        'drawing is available to o'
+        return not self.locked() or self.isowner(o)
+
+    def isowner(self, o):
+        'o owns the lock'
+        return self._owner is o
+
+    def locked(self):
+        'the lock is held'
+        return self._owner is not None
+
+
 
 class Widget:
     """
     OK, I couldn't resist; abstract base class for mpl GUI neutral
     widgets
-    """    
+    """
     drawon = True
     eventson = True
-        
-        
+
+
 
 
 class Button(Widget):
@@ -41,7 +76,7 @@ class Button(Widget):
     """
 
     def __init__(self, ax, label, image=None,
-                 color=0.85, hovercolor=0.95):
+                 color='0.85', hovercolor='0.95'):
         """
         ax is the Axes instance the button will be placed into
 
@@ -68,16 +103,16 @@ class Button(Widget):
         self.observers = {}
         self.ax = ax
 
-        
+
         ax.figure.canvas.mpl_connect('button_press_event', self._click)
         ax.figure.canvas.mpl_connect('motion_notify_event', self._motion)
         ax.set_navigate(False)
         ax.set_axis_bgcolor(color)
         ax.set_xticks([])
-        ax.set_yticks([])        
+        ax.set_yticks([])
         self.color = color
-        self.hovercolor = hovercolor        
-        
+        self.hovercolor = hovercolor
+
         self._lastcolor = color
 
     def _click(self, event):
@@ -95,7 +130,7 @@ class Button(Widget):
             self.ax.set_axis_bgcolor(c)
             self._lastcolor = c
             if self.drawon: self.ax.figure.canvas.draw()
-        
+
     def on_clicked(self, func):
         """
         When the button is clicked, call this func with event
@@ -104,33 +139,33 @@ class Button(Widget):
         """
         cid = self.cnt
         self.observers[cid] = func
-        self.cnt += 1        
+        self.cnt += 1
         return cid
 
     def disconnect(self, cid):
         'remove the observer with connection id cid'
         try: del self.observers[cid]
         except KeyError: pass
-        
-        
+
+
 
 class Slider(Widget):
     """
     A slider representing a floating point range
 
-    The following attributes are defined 
+    The following attributes are defined
       ax     : the slider axes.Axes instance
       val    : the current slider value
       vline  : a Line2D instance representing the initial value
       poly   : A patch.Polygon instance which is the slider
       valfmt : the format string for formatting the slider text
       label  : a text.Text instance, the slider label
-      closedmin : whether the slider is closed on the minimum 
+      closedmin : whether the slider is closed on the minimum
       closedmax : whether the slider is closed on the maximum
       slidermin : another slider - if not None, this slider must be > slidermin
       slidermax : another slider - if not None, this slider must be < slidermax
       dragging : allow for mouse dragging on slider
-      
+
     Call on_changed to connect to the slider event
     """
     def __init__(self, ax, label, valmin, valmax, valinit=0.5, valfmt='%1.2f',
@@ -141,7 +176,7 @@ class Slider(Widget):
 
         valinit -  the slider initial position
 
-        label - the slider label 
+        label - the slider label
 
         valfmt - used to format the slider value
 
@@ -157,16 +192,16 @@ class Slider(Widget):
         self.val = valinit
         self.valinit = valinit
         self.poly = ax.axvspan(valmin,valinit,0,1)
-        
+
         self.vline = ax.axvline(valinit,0,1, color='r', lw=1)
 
-        
+
         self.valfmt=valfmt
         ax.set_yticks([])
         ax.set_xlim((valmin, valmax))
         ax.set_xticks([])
         ax.set_navigate(False)
-        
+
         ax.figure.canvas.mpl_connect('button_press_event', self._update)
         if dragging:
             ax.figure.canvas.mpl_connect('motion_notify_event', self._update)
@@ -193,26 +228,26 @@ class Slider(Widget):
         if event.inaxes != self.ax: return
         val = event.xdata
         if not self.closedmin and val<=self.valmin: return
-        if not self.closedmax and val>=self.valmax: return        
-        
+        if not self.closedmax and val>=self.valmax: return
+
         if self.slidermin is not None:
             if val<=self.slidermin.val: return
 
         if self.slidermax is not None:
             if val>=self.slidermax.val: return
-            
-        self._set_val(val)
 
-    def _set_val(self, val):
+        self.set_val(val)
+
+    def set_val(self, val):
         self.poly.xy[-1] = val, 0
-        self.poly.xy[-2] = val, 1        
+        self.poly.xy[-2] = val, 1
         self.valtext.set_text(self.valfmt%val)
         if self.drawon: self.ax.figure.canvas.draw()
         self.val = val
         if not self.eventson: return
         for cid, func in self.observers.items():
             func(val)
-        
+
     def on_changed(self, func):
         """
         When the slider valud is changed, call this func with the new
@@ -222,7 +257,7 @@ class Slider(Widget):
         """
         cid = self.cnt
         self.observers[cid] = func
-        self.cnt += 1        
+        self.cnt += 1
         return cid
 
     def disconnect(self, cid):
@@ -233,7 +268,7 @@ class Slider(Widget):
     def reset(self):
         "reset the slider to the initial value if needed"
         if (self.val != self.valinit):
-            self._set_val(self.valinit)     
+            self.set_val(self.valinit)
 
 
 
@@ -265,7 +300,7 @@ class CheckButtons(Widget):
 
         ax.set_xticks([])
         ax.set_yticks([])
-        ax.set_navigate(False)        
+        ax.set_navigate(False)
 
         if len(labels)>1:
             dy = 1./(len(labels)+1)
@@ -273,7 +308,7 @@ class CheckButtons(Widget):
         else:
             dy = 0.25
             ys = [0.5]
-            
+
         cnt = 0
         axcolor = ax.get_axis_bgcolor()
 
@@ -306,7 +341,7 @@ class CheckButtons(Widget):
             self.lines.append((l1,l2))
             ax.add_patch(p)
             ax.add_line(l1)
-            ax.add_line(l2)            
+            ax.add_line(l2)
             cnt += 1
 
         ax.figure.canvas.mpl_connect('button_press_event', self._clicked)
@@ -325,14 +360,14 @@ class CheckButtons(Widget):
                 p.get_window_extent().contains(event.x, event.y) ):
                 l1, l2 = lines
                 l1.set_visible(not l1.get_visible())
-                l2.set_visible(not l2.get_visible())            
+                l2.set_visible(not l2.get_visible())
                 thist = t
                 break
         else:
             return
-            
 
-        if self.drawon: self.ax.figure.canvas.draw() 
+
+        if self.drawon: self.ax.figure.canvas.draw()
 
         if not self.eventson: return
         for cid, func in self.observers.items():
@@ -347,7 +382,7 @@ class CheckButtons(Widget):
         """
         cid = self.cnt
         self.observers[cid] = func
-        self.cnt += 1        
+        self.cnt += 1
         return cid
 
     def disconnect(self, cid):
@@ -380,11 +415,11 @@ class RadioButtons(Widget):
         activecolor is the color of the button when clicked
         """
         self.activecolor = activecolor
-        
+
 
         ax.set_xticks([])
         ax.set_yticks([])
-        ax.set_navigate(False)        
+        ax.set_navigate(False)
         dy = 1./(len(labels)+1)
         ys = linspace(1-dy, dy, len(labels))
         cnt = 0
@@ -401,11 +436,11 @@ class RadioButtons(Widget):
                 facecolor = activecolor
             else:
                 facecolor = axcolor
-                
+
             p = Circle(xy=(0.15, y), radius=0.05, facecolor=facecolor,
                        transform=ax.transAxes)
 
-            
+
             self.labels.append(t)
             self.circles.append(p)
             ax.add_patch(p)
@@ -417,7 +452,7 @@ class RadioButtons(Widget):
 
         self.cnt = 0
         self.observers = {}
-        
+
     def _clicked(self, event):
         if event.button !=1 : return
         if event.inaxes != self.ax: return
@@ -441,7 +476,7 @@ class RadioButtons(Widget):
 
 
 
-        if self.drawon: self.ax.figure.canvas.draw() 
+        if self.drawon: self.ax.figure.canvas.draw()
 
         if not self.eventson: return
         for cid, func in self.observers.items():
@@ -456,7 +491,7 @@ class RadioButtons(Widget):
         """
         cid = self.cnt
         self.observers[cid] = func
-        self.cnt += 1        
+        self.cnt += 1
         return cid
 
     def disconnect(self, cid):
@@ -465,7 +500,7 @@ class RadioButtons(Widget):
         except KeyError: pass
 
 
-                
+
 class SubplotTool(Widget):
     """
     A tool to adjust to subplot params of fig
@@ -484,7 +519,7 @@ class SubplotTool(Widget):
         class toolbarfmt:
             def __init__(self, slider):
                 self.slider = slider
-                
+
             def __call__(self, x, y):
                 fmt = '%s=%s'%(self.slider.label.get_text(), self.slider.valfmt)
                 return fmt%x
@@ -492,7 +527,7 @@ class SubplotTool(Widget):
         self.axleft = toolfig.add_subplot(711)
         self.axleft.set_title('Click on slider to adjust subplot param')
         self.axleft.set_navigate(False)
-        
+
         self.sliderleft = Slider(self.axleft, 'left', 0, 1, targetfig.subplotpars.left, closedmax=False)
         self.sliderleft.on_changed(self.funcleft)
 
@@ -501,23 +536,23 @@ class SubplotTool(Widget):
         self.axbottom.set_navigate(False)
         self.sliderbottom = Slider(self.axbottom, 'bottom', 0, 1, targetfig.subplotpars.bottom, closedmax=False)
         self.sliderbottom.on_changed(self.funcbottom)
-        
+
         self.axright = toolfig.add_subplot(713)
         self.axright.set_navigate(False)
         self.sliderright = Slider(self.axright, 'right', 0, 1, targetfig.subplotpars.right, closedmin=False)
         self.sliderright.on_changed(self.funcright)
-        
+
         self.axtop = toolfig.add_subplot(714)
         self.axtop.set_navigate(False)
         self.slidertop = Slider(self.axtop, 'top', 0, 1, targetfig.subplotpars.top, closedmin=False)
         self.slidertop.on_changed(self.functop)
 
-        
+
         self.axwspace = toolfig.add_subplot(715)
         self.axwspace.set_navigate(False)
         self.sliderwspace = Slider(self.axwspace, 'wspace', 0, 1, targetfig.subplotpars.wspace, closedmax=False)
         self.sliderwspace.on_changed(self.funcwspace)
-        
+
         self.axhspace = toolfig.add_subplot(716)
         self.axhspace.set_navigate(False)
         self.sliderhspace = Slider(self.axhspace, 'hspace', 0, 1, targetfig.subplotpars.hspace, closedmax=False)
@@ -529,7 +564,7 @@ class SubplotTool(Widget):
         self.sliderright.slidermin = self.sliderleft
         self.sliderbottom.slidermax = self.slidertop
         self.slidertop.slidermin = self.sliderbottom
-        
+
 
         bax = toolfig.add_axes([0.8, 0.05, 0.15, 0.075])
         self.buttonreset = Button(bax, 'Reset')
@@ -544,7 +579,7 @@ class SubplotTool(Widget):
             self.drawon = False
 
             # store the drawon state of each slider
-            bs = [] 
+            bs = []
             for slider in sliders:
                 bs.append(slider.drawon)
                 slider.drawon = False
@@ -604,7 +639,7 @@ class Cursor:
     the attributes
 
       horizOn =True|False: controls visibility of the horizontal line
-      vertOn =True|False: controls visibility of the horizontal line      
+      vertOn =True|False: controls visibility of the horizontal line
 
     And the visibility of the cursor itself with visible attribute
     """
@@ -631,69 +666,148 @@ class Cursor:
 
         self.background = None
         self.needclear = False
-        
-        
+
+
     def clear(self, event):
         'clear the cursor'
-        self.background = self.canvas.copy_from_bbox(self.ax.bbox)
+        if self.useblit:
+            self.background = self.canvas.copy_from_bbox(self.ax.bbox)
         self.linev.set_visible(False)
-        self.lineh.set_visible(False)        
+        self.lineh.set_visible(False)
 
     def onmove(self, event):
         'on mouse motion draw the cursor if visible'
         if event.inaxes != self.ax:
             self.linev.set_visible(False)
-            self.lineh.set_visible(False)        
+            self.lineh.set_visible(False)
 
-            if self.needclear:            
+            if self.needclear:
                 self.canvas.draw()
                 self.needclear = False
-            return 
+            return
         self.needclear = True
-        if not self.visible: return 
+        if not self.visible: return
         self.linev.set_xdata((event.xdata, event.xdata))
+
         self.lineh.set_ydata((event.ydata, event.ydata))
         self.linev.set_visible(self.visible and self.vertOn)
-        self.lineh.set_visible(self.visible and self.horizOn)        
+        self.lineh.set_visible(self.visible and self.horizOn)
+
         self._update()
-        
+
 
     def _update(self):
-        
+
         if self.useblit:
             if self.background is not None:
                 self.canvas.restore_region(self.background)
             self.ax.draw_artist(self.linev)
-            self.ax.draw_artist(self.lineh)            
+            self.ax.draw_artist(self.lineh)
             self.canvas.blit(self.ax.bbox)
         else:
+
             self.canvas.draw_idle()
 
         return False
 
-class HorizontalSpanSelector:
+class MultiCursor:
     """
-    Select a min/max range of the x axes for a matplotlib Axes
+    Provide a vertical line cursor shared between multiple axes
+
+    from matplotlib.widgets import MultiCursor
+    from pylab import figure, show, nx
+
+    t = nx.arange(0.0, 2.0, 0.01)
+    s1 = nx.sin(2*nx.pi*t)
+    s2 = nx.sin(4*nx.pi*t)
+    fig = figure()
+    ax1 = fig.add_subplot(211)
+    ax1.plot(t, s1)
+
+
+    ax2 = fig.add_subplot(212, sharex=ax1)
+    ax2.plot(t, s2)
+
+    multi = MultiCursor(fig.canvas, (ax1, ax2), color='r', lw=1)
+    show()
+
+    """
+    def __init__(self, canvas, axes, useblit=True, **lineprops):
+        self.canvas = canvas
+        self.axes = axes
+        xmin, xmax = axes[-1].get_xlim()
+        xmid = 0.5*(xmin+xmax)
+        self.lines = [ax.axvline(xmid, visible=False, **lineprops) for ax in axes]
+
+        self.visible = True
+        self.useblit = useblit
+        self.background = None
+        self.needclear = False
+
+        self.canvas.mpl_connect('motion_notify_event', self.onmove)
+        self.canvas.mpl_connect('draw_event', self.clear)
+
+
+    def clear(self, event):
+        'clear the cursor'
+        if self.useblit:
+            self.background = self.canvas.copy_from_bbox(self.canvas.figure.bbox)
+        for line in self.lines: line.set_visible(False)
+
+
+    def onmove(self, event):
+        if event.inaxes is None: return
+        if not self.canvas.widgetlock.available(self): return
+        self.needclear = True
+        if not self.visible: return
+
+        for line in self.lines:
+            line.set_xdata((event.xdata, event.xdata))
+            line.set_visible(self.visible)
+        self._update()
+
+
+    def _update(self):
+
+        if self.useblit:
+            if self.background is not None:
+                self.canvas.restore_region(self.background)
+            for ax, line in zip(self.axes, self.lines):
+                ax.draw_artist(line)
+            self.canvas.blit(self.canvas.figure.bbox)
+        else:
+
+            self.canvas.draw_idle()
+
+class SpanSelector:
+    """
+    Select a min/max range of the x or y axes for a matplotlib Axes
 
     Example usage:
 
       ax = subplot(111)
       ax.plot(x,y)
 
-      def onselect(xmin, xmax):
-      print xmin, xmax
-      span = HorizontalSpanSelector(ax, onselect)
+      def onselect(vmin, vmax):
+          print vmin, vmax
+      span = SpanSelector(ax, onselect, 'horizontal')
+
+      onmove_callback is an optional callback that will be called on mouse move
+      with the span range
 
     """
-    def __init__(self, ax, onselect, minspan=None, useblit=False, rectprops=None):
+
+    def __init__(self, ax, onselect, direction, minspan=None, useblit=False, rectprops=None, onmove_callback=None):
         """
         Create a span selector in ax.  When a selection is made, clear
         the span and call onselect with
 
-          onselect(xmin, xmax)
+          onselect(vmin, vmax)
 
         and clear the span.
 
+        direction must be 'horizontal' or 'vertical'
+
         If minspan is not None, ignore events smaller than minspan
 
         The span rect is drawn with rectprops; default
@@ -705,8 +819,11 @@ class HorizontalSpanSelector:
 
         """
         if rectprops is None:
-            rectprops = dict(facecolor='red', alpha=0.5)        
-            
+            rectprops = dict(facecolor='red', alpha=0.5)
+
+        assert direction in ['horizontal', 'vertical'], 'Must choose horizontal or vertical for direction'
+        self.direction = direction
+
         self.ax = ax
         self.visible = True
         self.canvas = ax.figure.canvas
@@ -720,52 +837,70 @@ class HorizontalSpanSelector:
 
         self.rectprops = rectprops
         self.onselect = onselect
+        self.onmove_callback = onmove_callback
         self.useblit = useblit
         self.minspan = minspan
 
-        trans = blend_xy_sep_transform(self.ax.transData, self.ax.transAxes)
+        # Needed when dragging out of axes
+        self.buttonDown = False
+        self.prev = (0, 0)
 
-        self.rect = Rectangle( (0,0), 0, 1,
+        if self.direction == 'horizontal':
+            trans = blend_xy_sep_transform(self.ax.transData, self.ax.transAxes)
+            w,h = 0,1
+        else:
+            trans = blend_xy_sep_transform(self.ax.transAxes, self.ax.transData)
+            w,h = 1,0
+        self.rect = Rectangle( (0,0), w, h,
                                transform=trans,
                                visible=False,
-                               **self.rectprops                               
+                               **self.rectprops
                                )
-        
+
         if not self.useblit: self.ax.add_patch(self.rect)
-        self.pressx = None
-        
+        self.pressv = None
+
     def update_background(self, event):
         'force an update of the background'
         if self.useblit:
             self.background = self.canvas.copy_from_bbox(self.ax.bbox)
 
-        
+
     def ignore(self, event):
         'return True if event should be ignored'
-        return  event.inaxes!=self.ax or not self.visible or event.button !=1 
+        return  event.inaxes!=self.ax or not self.visible or event.button !=1
 
     def press(self, event):
         'on button press event'
         if self.ignore(event): return
-        
+        self.buttonDown = True
+
         self.rect.set_visible(self.visible)
-        self.pressx = event.xdata
+        if self.direction == 'horizontal':
+            self.pressv = event.xdata
+        else:
+            self.pressv = event.ydata
         return False
 
 
     def release(self, event):
         'on button release event'
-        if self.pressx is None or self.ignore(event): return
+        if self.pressv is None or (self.ignore(event) and not self.buttonDown): return
+        self.buttonDown = False
 
         self.rect.set_visible(False)
         self.canvas.draw()
-        xmin = self.pressx
-        xmax = event.xdata
-        if xmin>xmax: xmin, xmax = xmax, xmin
-        span = xmax - xmin
+        vmin = self.pressv
+        if self.direction == 'horizontal':
+            vmax = event.xdata or self.prev[0]
+        else:
+            vmax = event.ydata or self.prev[1]
+
+        if vmin>vmax: vmin, vmax = vmax, vmin
+        span = vmax - vmin
         if self.minspan is not None and span<self.minspan: return
-        self.onselect(xmin, xmax)
-        self.pressx = None
+        self.onselect(vmin, vmax)
+        self.pressv = None
         return False
 
     def update(self):
@@ -782,12 +917,269 @@ class HorizontalSpanSelector:
 
     def onmove(self, event):
         'on motion notify event'
-        if self.pressx is None or self.ignore(event): return
-        x,y = event.xdata, event.ydata
+        if self.pressv is None or self.ignore(event): return
+        x, y = event.xdata, event.ydata
+        self.prev = x, y
+        if self.direction == 'horizontal':
+            v = x
+        else:
+            v = y
 
-        minx, maxx = x, self.pressx
-        if minx>maxx: minx, maxx = maxx, minx
-        self.rect.xy[0] = minx
-        self.rect.set_width(maxx-minx)            
+        minv, maxv = v, self.pressv
+        if minv>maxv: minv, maxv = maxv, minv
+        if self.direction == 'horizontal':
+            self.rect.xy[0] = minv
+            self.rect.set_width(maxv-minv)
+        else:
+            self.rect.xy[1] = minv
+            self.rect.set_height(maxv-minv)
+
+        if self.onmove_callback is not None:
+            vmin = self.pressv
+            if self.direction == 'horizontal':
+                vmax = event.xdata or self.prev[0]
+            else:
+                vmax = event.ydata or self.prev[1]
+
+            if vmin>vmax: vmin, vmax = vmax, vmin
+            self.onmove_callback(vmin, vmax)
+            
         self.update()
         return False
+
+# For backwards compatibility only!
+class HorizontalSpanSelector(SpanSelector):
+    def __init__(self, ax, onselect, **kwargs):
+        import warnings
+        warnings.warn('Use SpanSelector instead!', DeprecationWarning)
+        SpanSelector.__init__(self, ax, onselect, 'horizontal', **kwargs)
+
+class RectangleSelector:
+    """
+    Select a min/max range of the x axes for a matplotlib Axes
+
+    Example usage:
+
+      ax = subplot(111)
+      ax.plot(x,y)
+
+      def onselect(eclick, erelease):
+          'eclick and erelease are matplotlib events at press and release'
+          print 'startposition : (%f,%f)'%(eclick.xdata, eclick.ydata)
+          print 'endposition   : (%f,%f)'%(erelease.xdata, erelease.ydata)
+          print 'used button   : ', eclick.button
+
+      span = Selector(ax, onselect,drawtype='box')
+      show()
+
+    """
+    def __init__(self, ax, onselect, drawtype='box',
+                 minspanx=None, minspany=None, useblit=False,
+                 lineprops=None, rectprops=None):
+
+        """
+        Create a selector in ax.  When a selection is made, clear
+        the span and call onselect with
+
+          onselect(pos_1, pos_2)
+
+        and clear the drawn box/line. There pos_i are arrays of length 2
+        containing the x- and y-coordinate.
+
+        If minspanx is not None then events smaller than minspanx
+        in x direction are ignored(it's the same for y).
+
+        The rect is drawn with rectprops; default
+          rectprops = dict(facecolor='red', edgecolor = 'black',
+                           alpha=0.5, fill=False)
+
+        The line is drawn with lineprops; default
+          lineprops = dict(color='black', linestyle='-',
+                           linewidth = 2, alpha=0.5)
+
+        Use type if you want the mouse to draw a line, a box or nothing
+        between click and actual position ny setting
+        drawtype = 'line', drawtype='box' or drawtype = 'none'.
+
+
+        """
+        self.ax = ax
+        self.visible = True
+        self.canvas = ax.figure.canvas
+        self.canvas.mpl_connect('motion_notify_event', self.onmove)
+        self.canvas.mpl_connect('button_press_event', self.press)
+        self.canvas.mpl_connect('button_release_event', self.release)
+        self.canvas.mpl_connect('draw_event', self.update_background)
+
+        self.to_draw = None
+        self.background = None
+
+        if drawtype == 'none':
+            drawtype = 'line'                        # draw a line but make it
+            self.visible = False                     # invisible
+
+        if drawtype == 'box':
+            if rectprops is None:
+                rectprops = dict(facecolor='white', edgecolor = 'black',
+                                 alpha=0.5, fill=False)
+            self.rectprops = rectprops
+            self.to_draw = Rectangle((0,0), 0, 1,visible=False,**self.rectprops)
+            self.ax.add_patch(self.to_draw)
+        if drawtype == 'line':
+            if lineprops is None:
+                lineprops = dict(color='black', linestyle='-',
+                                 linewidth = 2, alpha=0.5)
+            self.lineprops = lineprops
+            self.to_draw = Line2D([0,0],[0,0],visible=False,**self.lineprops)
+            self.ax.add_line(self.to_draw)
+
+        self.onselect = onselect
+        self.useblit = useblit
+        self.minspanx = minspanx
+        self.minspany = minspany
+        self.drawtype = drawtype
+        # will save the data (position at mouseclick)
+        self.eventpress = None
+        # will save the data (pos. at mouserelease)
+        self.eventrelease = None
+
+    def update_background(self, event):
+        'force an update of the background'
+        if self.useblit:
+            self.background = self.canvas.copy_from_bbox(self.ax.bbox)
+
+
+    def ignore(self, event):
+        'return True if event should be ignored'
+        # If no button was pressed yet ignore the event if it was out
+        # of the axes
+        if self.eventpress == None:
+            return event.inaxes!= self.ax
+
+        # If a button was pressed, check if the release-button is the
+        # same.
+        return  (event.inaxes!=self.ax or
+                 event.button != self.eventpress.button)
+
+    def press(self, event):
+        'on button press event'
+        # Is the correct button pressed within the correct axes?
+        if self.ignore(event): return
+
+
+        # make the drawed box/line visible get the click-coordinates,
+        # button, ...
+        self.to_draw.set_visible(self.visible)
+        self.eventpress = event
+        return False
+
+
+    def release(self, event):
+        'on button release event'
+        if self.eventpress is None or self.ignore(event): return
+        # make the box/line invisible again
+        self.to_draw.set_visible(False)
+        self.canvas.draw()
+        # release coordinates, button, ...
+        self.eventrelease = event
+        xmin, ymin = self.eventpress.xdata, self.eventpress.ydata
+        xmax, ymax = self.eventrelease.xdata, self.eventrelease.ydata
+        # calculate dimensions of box or line get values in the right
+        # order
+        if xmin>xmax: xmin, xmax = xmax, xmin
+        if ymin>ymax: ymin, ymax = ymax, ymin
+
+
+
+        spanx = xmax - xmin
+        spany = ymax - ymin
+        xproblems = self.minspanx is not None and spanx<self.minspanx
+        yproblems = self.minspany is not None and spany<self.minspany
+        if (self.drawtype=='box')  and (xproblems or  yproblems):
+            """Box to small"""     # check if drawed distance (if it exists) is
+            return                 # not to small in neither x nor y-direction
+        if (self.drawtype=='line') and (xproblems and yproblems):
+            """Line to small"""    # check if drawed distance (if it exists) is
+            return                 # not to small in neither x nor y-direction
+        self.onselect(self.eventpress, self.eventrelease)
+                                              # call desired function
+        self.eventpress = None                # reset the variables to their
+        self.eventrelease = None              #   inital values
+        return False
+
+    def update(self):
+        'draw using newfangled blit or oldfangled draw depending on useblit'
+        if self.useblit:
+            if self.background is not None:
+                self.canvas.restore_region(self.background)
+            self.ax.draw_artist(self.to_draw)
+            self.canvas.blit(self.ax.bbox)
+        else:
+            self.canvas.draw_idle()
+        return False
+
+
+    def onmove(self, event):
+        'on motion notify event if box/line is wanted'
+        if self.eventpress is None or self.ignore(event): return
+        x,y = event.xdata, event.ydata            # actual position (with
+                                                  #   (button still pressed)
+        if self.drawtype == 'box':
+            minx, maxx = self.eventpress.xdata, x # click-x and actual mouse-x
+            miny, maxy = self.eventpress.ydata, y # click-y and actual mouse-y
+            if minx>maxx: minx, maxx = maxx, minx # get them in the right order
+            if miny>maxy: miny, maxy = maxy, miny
+            self.to_draw.xy[0] = minx             # set lower left of box
+            self.to_draw.xy[1] = miny
+            self.to_draw.set_width(maxx-minx)     # set width and height of box
+            self.to_draw.set_height(maxy-miny)
+            self.update()
+            return False
+        if self.drawtype == 'line':
+            self.to_draw.set_data([self.eventpress.xdata, x],
+                                  [self.eventpress.ydata, y])
+            self.update()
+            return False
+
+class Lasso(Widget):
+    def __init__(self, ax, xy, callback=None, useblit=True):
+        self.axes = ax
+        self.figure = ax.figure
+        self.canvas = self.figure.canvas
+        self.useblit = useblit
+        if useblit:
+            self.background = self.canvas.copy_from_bbox(self.axes.bbox)
+
+        x, y = xy
+        self.verts = [(x,y)]
+        self.line = Line2D([x], [y], linestyle='-', color='black', lw=2)
+        self.axes.add_line(self.line)
+        self.callback = callback
+        self.cids = []
+        self.cids.append(self.canvas.mpl_connect('button_release_event', self.onrelease))
+        self.cids.append(self.canvas.mpl_connect('motion_notify_event', self.onmove))
+
+    def onrelease(self, event):
+        if self.verts is not None:
+            self.verts.append((event.xdata, event.ydata))
+            if len(self.verts)>2:
+                self.callback(self.verts)
+            self.axes.lines.remove(self.line)
+        self.verts = None
+        for cid in self.cids:
+            self.canvas.mpl_disconnect(cid)
+        
+    def onmove(self, event):
+        if self.verts is None: return 
+        if event.inaxes != self.axes: return
+        if event.button!=1: return 
+        self.verts.append((event.xdata, event.ydata))
+
+        self.line.set_data(zip(*self.verts))
+
+        if self.useblit:
+            self.canvas.restore_region(self.background)
+            self.axes.draw_artist(self.line)
+            self.canvas.blit(self.axes.bbox)
+        else:
+            self.canvas.draw_idle()
