@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
  
 from pylab import *
 
@@ -7,7 +7,7 @@ dt = 0.001
 t = arange(0.0, 10.0, dt)
 r = exp(-t[:1000]/0.05)               # impulse response
 x = randn(len(t))
-s = convolve(x,r,mode=2)[:len(x)]*dt  # colored noise
+s = conv(x,r)[:len(x)]*dt  # colored noise
 
 # the main axes is subplot(111) by default
 plot(t, s)
