@@ -1,11 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
-This is used to drive many of the examples across the image backends,
-for regression testing, and comparing backend efficiency
+This is used to drive many of the examples across the backends, for
+regression testing, and comparing backend efficiency
 """
 
 from __future__ import division
-import os, time
+import os, time, sys
 files = (
     'alignment_test.py',
     'arctest.py',
@@ -19,7 +19,7 @@ files = (
     'custom_ticker1.py',
     'customize_rc.py',
     'date_demo1.py',
-    'date_demo2.py',    
+    'date_demo2.py',
     'figimage_demo.py',
     'figlegend_demo.py',
     'figtext.py',
@@ -36,11 +36,13 @@ files = (
     'legend_demo.py',
     'legend_demo2.py',
     'line_collection.py',
+    'line_collection2.py',
     'line_styles.py',
     'log_demo.py',
     'log_test.py',
     'major_minor_demo1.py',
-    'major_minor_demo2.py',     
+    'major_minor_demo2.py',
+    'masked_demo.py',
     'mathtext_demo.py',
     'mri_with_eeg.py',
     'multiple_figs_demo.py',
@@ -51,21 +53,23 @@ files = (
     'polar_demo.py',
     'polar_scatter.py',
     'psd_demo.py',
+    'quiver_demo.py',
     'scatter_demo.py',
     'scatter_demo2.py',
     'simple_plot.py',
     'specgram_demo.py',
     'stock_demo.py',
     'subplot_demo.py',
-#    'set_and_get.py',    
+#    'set_and_get.py',
     'table_demo.py',
     'text_handles.py',
     'text_rotation.py',
     'text_themes.py',
-#    'tex_demo.py',    
+#    'tex_demo.py',
     'two_scales.py',
     'unicode_demo.py',
     'vline_demo.py',
+    'xcorr_demo.py',
     'zorder_demo.py',
     )
 
@@ -73,7 +77,7 @@ files = (
 #tests known to fail on python22 (require datetime)
 fail22  = (
     'date_demo1.py',
-    'date_demo2.py',    
+    'date_demo2.py',
     'finance_demo.py',
     )
 
@@ -84,18 +88,15 @@ failbackend = dict(
     SVG = ('tex_demo.py,'),
     )
 
-def drive(backend, python='python2.4'):
+def drive(backend, python='python'):
 
     exclude = failbackend.get(backend, [])
-    
+
     for fname in files:
         if fname in exclude:
             print '\tSkipping %s, known to fail on backend: %s'%backend
             continue
-            
-        if python=='python2.2' and fname in fail22:
-            print '\tSkipping %s, known to fail on python2.2'%fname
-            continue
+
         print '\tdriving %s' % fname
         basename, ext = os.path.splitext(fname)
         outfile = basename + '_%s'%backend
@@ -106,6 +107,7 @@ def drive(backend, python='python2.4'):
             'from __future__ import division\n',
             'import matplotlib\n',
             'matplotlib.use("%s")\n' % backend,
+            'from pylab import savefig\n',
             ))
         for line in file(fname):
             line_lstrip = line.lstrip()
@@ -128,10 +130,19 @@ def drive(backend, python='python2.4'):
 if __name__ == '__main__':
     times = {}
     # backends = ['Agg', 'Cairo', 'GDK', 'PS', 'SVG', 'Template']
-    backends = ['Agg', 'PS', 'SVG', 'Template']
+    #backends = ['Agg', 'PS', 'SVG', 'Template']
     # backends = [ 'GTK', 'WX', 'TkAgg']
-    backends = ['Agg', 'PS', 'SVG', 'Template']
-    python = 'python2.4'
+    default_backends = ['Agg', 'PS', 'SVG', 'Template']
+    #default_backends = ['Agg']
+    #backends = ['Agg']
+    if sys.platform == 'win32':
+        python = r'c:\Python24\python.exe'
+    else:
+        python = 'python'
+    if sys.argv[1:]:
+        backends = [b for b in sys.argv[1:] if b in default_backends]
+    else:
+        backends = default_backends
     for backend in backends:
         print 'testing %s' % backend
         t0 = time.time()
