@@ -1,9 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
 Compute the cross spectral density of two signals
 """
 from __future__ import division
-import sys
 from pylab import *
 
 dt = 0.01
