@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 
 """
 You can precisely specify dashes with an on/off ink rect sequence in
@@ -10,5 +10,5 @@ dashes = [5,2,10,5] # 5 points on, 2 off, 3 on, 1 off
 
 l, = plot(arange(20), '--')
 l.set_dashes(dashes)
-#savefig('dash_control')
+savefig('dash_control')
 show()
