@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
 Show how to use an rrule instance to make a custom date ticker - here
 we put a tick mark on every 5th easter
@@ -28,5 +28,5 @@ ax.xaxis.set_major_formatter(formatter)
 labels = ax.get_xticklabels()
 setp(labels, rotation=30, fontsize=10)
 
-
+savefig('date_demo_rrule')
 show()
