@@ -1,19 +1,20 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
 show how to add a matplotlib FigureCanvasGTK or FigureCanvasGTKAgg widget to a
 gtk.Window
 """
 
+import gtk
+
 from matplotlib.axes import Subplot
 from matplotlib.figure import Figure
 from matplotlib.numerix import arange, sin, pi
 
+# uncomment to select /GTK/GTKAgg/GTKCairo
+from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
+#from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
+#from matplotlib.backends.backend_gtkcairo import FigureCanvasGTKCairo as FigureCanvas
 
-# switch comments for gtk over gtkagg
-#from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
-from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
-
-import gtk
 
 win = gtk.Window()
 win.connect("destroy", lambda x: gtk.main_quit())
@@ -26,10 +27,8 @@ t = arange(0.0,3.0,0.01)
 s = sin(2*pi*t)
 a.plot(t,s)
 
-
 canvas = FigureCanvas(f)  # a gtk.DrawingArea
 win.add(canvas)
 
-
 win.show_all()
 gtk.main()
