@@ -1,17 +1,18 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
 demonstrate adding a FigureCanvasGTK/GTKAgg widget to a gtk.ScrolledWindow
 """
 
+import gtk
+
 from matplotlib.axes import Subplot
 from matplotlib.figure import Figure
 from matplotlib.numerix import arange, sin, pi
 
-# switch comments for gtk over gtkagg
-from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
-#from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
-
-import gtk
+# uncomment to select /GTK/GTKAgg/GTKCairo
+#from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
+from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
+#from matplotlib.backends.backend_gtkcairo import FigureCanvasGTKCairo as FigureCanvas
 
 win = gtk.Window()
 win.connect("destroy", lambda x: gtk.main_quit())
@@ -29,7 +30,7 @@ win.add (sw)
 # A scrolled window border goes outside the scrollbars and viewport
 sw.set_border_width (10)
 # policy: ALWAYS, AUTOMATIC, NEVER
-sw.set_policy (hscrollbar_policy=gtk.POLICY_AUTOMATIC, 
+sw.set_policy (hscrollbar_policy=gtk.POLICY_AUTOMATIC,
                vscrollbar_policy=gtk.POLICY_ALWAYS)
 
 canvas = FigureCanvas(f)  # a gtk.DrawingArea
