@@ -1,9 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 from pylab import *
 from matplotlib.dates import  DateFormatter, WeekdayLocator, HourLocator, \
      DayLocator, MONDAY, timezone
 from matplotlib.finance import quotes_historical_yahoo, candlestick,\
-     plot_day_summary
+     plot_day_summary, candlestick2
 
 import datetime
 
@@ -21,7 +21,9 @@ quotes = quotes_historical_yahoo(
 if not quotes:
     raise SystemExit
 
-ax = subplot(111)
+fig = figure()
+fig.subplots_adjust(bottom=0.2)
+ax = fig.add_subplot(111)
 ax.xaxis.set_major_locator(mondays)
 ax.xaxis.set_minor_locator(alldays)
 ax.xaxis.set_major_formatter(weekFormatter)
@@ -29,7 +31,10 @@ ax.xaxis.set_major_formatter(weekFormatter)
 
 #plot_day_summary(ax, quotes, ticksize=3)
 candlestick(ax, quotes, width=0.6)
-set( gca().get_xticklabels(), rotation=45, horizontalalignment='right')
+
+ax.xaxis_date()
+ax.autoscale_view()
+setp( gca().get_xticklabels(), rotation=45, horizontalalignment='right')
 
 show()
 
