@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 # Thanks to Charles Twardy for this example
 #
 #See http://matplotlib.sf.net/examples/legend_demo2.py for an example
@@ -36,10 +36,10 @@ frame  = leg.get_frame()  # the patch.Rectangle instance surrounding the legend
 
 # see text.Text, lines.Line2D, and patches.Rectangle for more info on
 # the settable properties of lines, text, and rectangles
-frame.set_facecolor(0.80)     # set the frame face color to light gray
+frame.set_facecolor('0.80')      # set the frame face color to light gray
 setp(ltext, fontsize='small')    # the legend text fontsize
-setp(llines, linewidth=1.5) # the legend linewidth
-#leg.draw_frame(False)         # don't draw the legend frame
+setp(llines, linewidth=1.5)      # the legend linewidth
+#leg.draw_frame(False)           # don't draw the legend frame
 #savefig('legend_demo')
 show()
 
