@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
 See pcolor_demo2 for an alternative way of generating pcolor plots
 using imshow that is likely faster for large grids
@@ -22,7 +22,7 @@ Z = func3(X, Y)
 pcolor(X, Y, Z, shading='flat')
 colorbar()
 axis([-3,3,-3,3])
-#savefig('pcolor_demo')
+savefig('pcolor_demo')
 show()
 
     
