@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
 Make a pie chart - see
 http://matplotlib.sf.net/matplotlib.pylab.html#-pie for the docstring.
@@ -30,6 +30,6 @@ figure(2, figsize=(8,8))
 explode=(0, 0.05, 0, 0)
 pie(fracs, explode=explode, labels=labels, autopct='%1.1f%%', shadow=True)
 
-#savefig('pie_demo')
+savefig('pie_demo')
 show()
     
