@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 # print png to standard out
 # usage: python print_stdout.py > somefile.png
 import sys
@@ -8,5 +8,5 @@ from pylab import *
 
 plot([1,2,3])
 
-#savefig(sys.stdout)
+savefig(sys.stdout)
 show()
