@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 import matplotlib
 matplotlib.use('PS')
 from pylab import *
@@ -15,10 +15,7 @@ t3 = arange(0.0, 2.0, 0.01)
 figure(1)
 subplot(211)
 l = plot(t1, f(t1), 'k^')
-set(l, 'markerfacecolor', 'k')
-set(l, 'markeredgecolor', 'r')
-#set(l, 'linewidth', 3)
-#set(l, 'markersize', 12)
+setp(l, markerfacecolor='k', markeredgecolor='r')
 title('A tale of 2 subplots', fontsize=14, fontname='Courier')
 ylabel('Signal 1', fontsize=12)
 subplot(212)
