@@ -1,9 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 from pylab import *
 
 x = linspace(0.1, 2*pi, 10)
 markerline, stemlines, baseline = stem(x, cos(x), '-.')
-set(markerline, 'markerfacecolor', 'b')
-set(baseline, 'color','r', 'linewidth', 2) 
+setp(markerline, 'markerfacecolor', 'b')
+setp(baseline, 'color','r', 'linewidth', 2) 
 
 show()
