@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 
 from matplotlib.ticker import MultipleLocator
 from pylab import *
@@ -8,10 +8,9 @@ d1, p1, d2, p2 = get_two_stock_data()
 
 ax = subplot(111)
 lines = plot(d1, p1, 'bs', d2, p2, 'go')
-setp(lines, 'data_clipping', True)
 xlabel('Days')
 ylabel('Normalized price')
-xlim(0, 3) 
+xlim(0, 3)
 ax.xaxis.set_major_locator(MultipleLocator(1))
 
 title('INTC vs AAPL')
