@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
 You can use TeX to render all of your matplotlib text if the rc
 parameter text.usetex is set.  This works currently on the agg and ps
@@ -23,12 +23,12 @@ t = arange(0.0, 1.0+0.01, 0.01)
 s = cos(2*2*pi*t)+2
 plot(t, s)
 
-xlabel(r'\bf{time (s)}')
-ylabel(r'\it{voltage (mV)}',fontsize=16)
+xlabel(r'\textbf{time (s)}')
+ylabel(r'\textit{voltage (mV)}',fontsize=16)
 title(r"\TeX\ is Number $\displaystyle\sum_{n=1}^\infty\frac{-e^{i\pi}}{2^n}$!", 
       fontsize=16, color='r')
 grid(True)
-#savefig('tex_demo.ps')
+savefig('tex_demo')
 
 
 show()
