@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 #Controlling the properties of axis text using handles
 
 # See examples/text_themes.py for a more elegant, pythonic way to control
@@ -24,7 +24,7 @@ ttext = title('Fun with text!')
 ytext = ylabel('Damped oscillation')
 xtext = xlabel('time (s)')
 
-set(ttext, size='large', color='r', style='italic')
-set(xtext, size='medium', name='courier', weight='bold', color='g')
-set(ytext, size='medium', name='helvetica', weight='light', color='b')
+setp(ttext, size='large', color='r', style='italic')
+setp(xtext, size='medium', name='courier', weight='bold', color='g')
+setp(ytext, size='medium', name='helvetica', weight='light', color='b')
 show()
