@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2.5
 """
 
 Demonstrate how to do two plots on the same axes with different left
@@ -34,5 +34,5 @@ ax2 = twinx()
 s2 = sin(2*pi*t)
 plot(t, s2, 'r.')
 ylabel('sin')
-
+ax2.yaxis.tick_right()
 show()
