@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python2.5
 # -*- coding: utf-8 -*-
 
 from pylab import *
@@ -9,5 +9,5 @@ ylabel( unicode('Andr\xc3\xa9 was here!', 'utf-8') )
 text( 0.5, 2.5, unicode('Institut fr Festkrperphysik', 'latin-1'), rotation=45)
 text( 1, 1.5, u'AVA (check kerning)')  
 
-#savefig('test.ps')
+savefig('test.svg')
 show()
